from oauth2.test import unittest
from mock import patch
from oauth2.datatype import AccessToken, Client
from oauth2.error import RedirectUriUnknown


def mock_time():
    return 1000


class AccessTokenTestCase(unittest.TestCase):
    @patch("time.time", mock_time)
    def test_expires_in_expired(self):
        access_token = AccessToken(client_id="abc",
                                   grant_type="client_credentials",
                                   token="def", expires_at=999)

        self.assertEqual(access_token.expires_in, 0)

    @patch("time.time", mock_time)
    def test_expires_in_not_expired(self):
        access_token = AccessToken(client_id="abc",
                                   grant_type="client_credentials",
                                   token="def", expires_at=1100)

        self.assertEqual(access_token.expires_in, 100)

    def test_is_expired_expired_at_not_set(self):
        access_token = AccessToken(client_id="abc",
                                   grant_type="client_credentials",
                                   token="def")

        self.assertFalse(access_token.is_expired())


class ClientTestCase(unittest.TestCase):
    def test_redirect_uri(self):
        client = Client(identifier="abc", secret="xyz",
                        redirect_uris=["http://callback"])

        self.assertEqual(client.redirect_uri, "http://callback")
        client.redirect_uri = "http://callback"
        self.assertEqual(client.redirect_uri, "http://callback")

        with self.assertRaises(RedirectUriUnknown):
            client.redirect_uri = "http://another.callback"

    def test_response_type_supported(self):
        client = Client(identifier="abc", secret="xyz",
                        authorized_grants=["test_grant"])

        self.assertTrue(client.grant_type_supported("test_grant"))
        self.assertFalse(client.grant_type_supported("unknown_grant"))

    def test_response_type_supported(self):
        client = Client(identifier="abc", secret="xyz",
                        authorized_response_types=["test_response_type"])

        self.assertTrue(client.response_type_supported("test_response_type"))
        self.assertFalse(client.response_type_supported("unknown"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Classes for handling a HTTP request/response flow.

.. versionchanged:: 1.0.0
   Moved from package ``oauth2.web`` to ``oauth2.web.wsgi``.
"""

from oauth2.compatibility import parse_qs


class Request(object):
    """
    Contains data of the current HTTP request.
    """
    def __init__(self, env):
        """
        :param env: Wsgi environment
        """
        self.method = env["REQUEST_METHOD"]
        self.query_params = {}
        self.query_string = env["QUERY_STRING"]
        self.path = env["PATH_INFO"]
        self.post_params = {}
        self.env_raw = env

        for param, value in parse_qs(env["QUERY_STRING"]).items():
            self.query_params[param] = value[0]

        if (self.method == "POST"
            and env["CONTENT_TYPE"].startswith("application/x-www-form-urlencoded")):
            self.post_params = {}
            content = env['wsgi.input'].read(int(env['CONTENT_LENGTH']))
            post_params = parse_qs(content)

            for param, value in post_params.items():
                decoded_param = param.decode('utf-8')
                decoded_value = value[0].decode('utf-8')
                self.post_params[decoded_param] = decoded_value

    def get_param(self, name, default=None):
        """
        Returns a param of a GET request identified by its name.
        """
        try:
            return self.query_params[name]
        except KeyError:
            return default

    def post_param(self, name, default=None):
        """
        Returns a param of a POST request identified by its name.
        """
        try:
            return self.post_params[name]
        except KeyError:
            return default

    def header(self, name, default=None):
        """
        Returns the value of the HTTP header identified by `name`.
        """
        wsgi_header = "HTTP_{0}".format(name.upper())

        try:
            return self.env_raw[wsgi_header]
        except KeyError:
            return default


class Application(object):
    """
    Implements WSGI.

    .. versionchanged:: 1.0.0
       Renamed from ``Server`` to ``Application``.
    """
    HTTP_CODES = {200: "200 OK",
                  301: "301 Moved Permanently",
                  302: "302 Found",
                  400: "400 Bad Request",
                  401: "401 Unauthorized",
                  404: "404 Not Found"}

    def __init__(self, provider, authorize_uri="/authorize", env_vars=None,
                 request_class=Request, token_uri="/token"):
        self.authorize_uri = authorize_uri
        self.env_vars = env_vars
        self.request_class = request_class
        self.provider = provider
        self.token_uri = token_uri

        self.provider.authorize_path = authorize_uri
        self.provider.token_path = token_uri

    def __call__(self, env, start_response):
        environ = {}

        if (env["PATH_INFO"] != self.authorize_uri
            and env["PATH_INFO"] != self.token_uri):
            start_response("404 Not Found",
                           [('Content-type', 'text/html')])
            return [b"Not Found"]

        request = self.request_class(env)

        if isinstance(self.env_vars, list):
            for varname in self.env_vars:
                if varname in env:
                    environ[varname] = env[varname]

        response = self.provider.dispatch(request, environ)

        start_response(self.HTTP_CODES[response.status_code],
                       list(response.headers.items()))

        return [response.body.encode('utf-8')]



from oauth2.test import unittest
from mock import Mock
from oauth2.web import Response
from oauth2.web.wsgi import Request, Application
from oauth2 import Provider


class RequestTestCase(unittest.TestCase):
    def test_initialization_no_post_data(self):
        request_method = "TEST"
        query_string = "foo=bar&baz=buz"

        environment = {"REQUEST_METHOD": request_method,
                       "QUERY_STRING": query_string,
                       "PATH_INFO": "/"}

        request = Request(environment)

        self.assertEqual(request.method, request_method)
        self.assertEqual(request.query_params, {"foo": "bar", "baz": "buz"})
        self.assertEqual(request.query_string, query_string)
        self.assertEqual(request.post_params, {})

    def test_initialization_with_post_data(self):
        content_length = "42"
        request_method = "POST"
        query_string = ""
        content = "foo=bar&baz=buz".encode('utf-8')

        wsgi_input_mock = Mock(spec=["read"])
        wsgi_input_mock.read.return_value = content

        environment = {"CONTENT_LENGTH": content_length,
                       "CONTENT_TYPE": "application/x-www-form-urlencoded",
                       "REQUEST_METHOD": request_method,
                       "QUERY_STRING": query_string,
                       "PATH_INFO": "/",
                       "wsgi.input": wsgi_input_mock}

        request = Request(environment)

        wsgi_input_mock.read.assert_called_with(int(content_length))
        self.assertEqual(request.method, request_method)
        self.assertEqual(request.query_params, {})
        self.assertEqual(request.query_string, query_string)
        self.assertEqual(request.post_params, {"foo": "bar", "baz": "buz"})

    def test_get_param(self):
        request_method = "TEST"
        query_string = "foo=bar&baz=buz"

        environment = {"REQUEST_METHOD": request_method,
                       "QUERY_STRING": query_string,
                       "PATH_INFO": "/a-url"}

        request = Request(environment)

        result = request.get_param("foo")

        self.assertEqual(result, "bar")

        result_default = request.get_param("na")

        self.assertEqual(result_default, None)

    def test_post_param(self):
        content_length = "42"
        request_method = "POST"
        query_string = ""
        content = "foo=bar&baz=buz".encode('utf-8')

        wsgi_input_mock = Mock(spec=["read"])
        wsgi_input_mock.read.return_value = content

        environment = {"CONTENT_LENGTH": content_length,
                       "CONTENT_TYPE": "application/x-www-form-urlencoded",
                       "REQUEST_METHOD": request_method,
                       "QUERY_STRING": query_string,
                       "PATH_INFO": "/",
                       "wsgi.input": wsgi_input_mock}

        request = Request(environment)

        result = request.post_param("foo")

        self.assertEqual(result, "bar")

        result_default = request.post_param("na")

        self.assertEqual(result_default, None)

        wsgi_input_mock.read.assert_called_with(int(content_length))

    def test_header(self):
        environment = {"REQUEST_METHOD": "GET",
                       "QUERY_STRING": "",
                       "PATH_INFO": "/",
                       "HTTP_AUTHORIZATION": "Basic abcd"}

        request = Request(env=environment)

        self.assertEqual(request.header("authorization"), "Basic abcd")
        self.assertIsNone(request.header("unknown"))
        self.assertEqual(request.header("unknown", default=0), 0)


class ServerTestCase(unittest.TestCase):
    def test_call(self):
        body = "body"
        headers = {"header": "value"}
        path = "/authorize"
        status_code = 200
        http_code = "200 OK"

        environment = {"PATH_INFO": path, "myvar": "value"}

        request_mock = Mock(spec=Request)
        request_class_mock = Mock(return_value=request_mock)

        response_mock = Mock(spec=Response)
        response_mock.body = body
        response_mock.headers = headers
        response_mock.status_code = status_code

        provider_mock = Mock(spec=Provider)
        provider_mock.dispatch.return_value = response_mock

        start_response_mock = Mock()

        wsgi = Application(provider=provider_mock, authorize_uri=path,
                           request_class=request_class_mock,
                           env_vars=["myvar"])
        result = wsgi(environment, start_response_mock)

        request_class_mock.assert_called_with(environment)
        provider_mock.dispatch.assert_called_with(request_mock,
                                                  {"myvar": "value"})
        start_response_mock.assert_called_with(http_code,
                                               list(headers.items()))
        self.assertEqual(result, [body.encode('utf-8')])



from functools import wraps
from mock import Mock, call
from oauth2.datatype import AccessToken, AuthorizationCode, Client
from oauth2.error import AccessTokenNotFound, ClientNotFoundError, \
    AuthCodeNotFound
from oauth2.store.dbapi.mysql import MysqlAccessTokenStore, MysqlAuthCodeStore, \
    MysqlClientStore
from oauth2.test import unittest


access_token_stores = [MysqlAccessTokenStore]

auth_code_stores = [MysqlAuthCodeStore]

client_stores = [MysqlClientStore]


class with_classes(object):
    def __init__(self, classes):
        self.classes = classes

    def __call__(self, func):
        @wraps(func)
        def wrapper(instance, *args, **kwargs):
            for klass in self.classes:
                func(instance, klass, *args, **kwargs)

        return wrapper


class StoreTestCase(unittest.TestCase):
    def _con_mock(self, cursor_mocks=None):
        connection_mock = Mock(spec=["close", "commit", "cursor"])

        if isinstance(cursor_mocks, list):
            connection_mock.cursor.side_effect = cursor_mocks

        return connection_mock

    def _cursor_mock(self):
        return Mock(spec=["close", "execute", "fetchone", "fetchall"])


class AccessTokenStoreTestCase(StoreTestCase):
    @with_classes(access_token_stores)
    def test_delete_refresh_token(self, store_class):
        refresh_token = "xyz789"

        cursor_mock = self._cursor_mock()
        cursor_mock.lastrowid = None

        connection_mock = self._con_mock()
        connection_mock.cursor.return_value = cursor_mock

        store = store_class(connection=connection_mock)
        store.delete_refresh_token(refresh_token)

        cursor_mock.execute.assert_called_with(
            store_class.delete_refresh_token_query,
            (refresh_token,))

        self.assertEqual(cursor_mock.close.call_count, 1)

        self.assertEqual(connection_mock.commit.call_count, 1)

    @with_classes(access_token_stores)
    def test_fetch_by_refresh_token(self, store_class):
        refresh_token = "xyz789"
        access_token_row_id = 1
        token_data = {"client_id": "abc", "grant_type": "test_grant",
                      "token": "abc123", "data": {"test": "data"},
                      "expires_at": 1000, "refresh_token": refresh_token,
                      "refresh_expires_at": 2000, "scopes": ["foo", "bar"],
                      "user_id": 1}

        data_cursor = self._cursor_mock()
        data_cursor.fetchall.return_value = [("test", "data")]

        scope_cursor = self._cursor_mock()
        scope_cursor.fetchall.return_value = [("foo",), ("bar",)]

        token_cursor = self._cursor_mock()

        token_cursor.fetchone.return_value = (access_token_row_id,
                                              token_data["client_id"],
                                              token_data["grant_type"],
                                              token_data["token"],
                                              token_data["expires_at"],
                                              token_data["refresh_token"],
                                              token_data["refresh_expires_at"],
                                              token_data["user_id"])

        connection_mock = self._con_mock([token_cursor, scope_cursor,
                                          data_cursor])

        store = store_class(connection=connection_mock)
        access_token = store.fetch_by_refresh_token(refresh_token)

        self.assertTrue(isinstance(access_token, AccessToken))
        self.assertEqual(access_token.client_id, token_data["client_id"])
        self.assertEqual(access_token.grant_type, token_data["grant_type"])
        self.assertEqual(access_token.token, token_data["token"])
        self.assertDictEqual(access_token.data, token_data["data"])
        self.assertEqual(access_token.expires_at, token_data["expires_at"])
        self.assertEqual(access_token.refresh_token, refresh_token)
        self.assertEqual(access_token.refresh_expires_at,
                         token_data["refresh_expires_at"])
        self.assertListEqual(access_token.scopes, token_data["scopes"])
        self.assertEqual(access_token.user_id, token_data["user_id"])

        token_cursor.execute.\
            assert_called_with(store_class.fetch_by_refresh_token_query,
                               (refresh_token,))

        scope_cursor.execute.\
            assert_called_with(store_class.fetch_scopes_by_access_token_query,
                               (access_token_row_id,))

        data_cursor.execute.\
            assert_called_with(store_class.fetch_data_by_access_token_query,
                               (access_token_row_id,))

    @with_classes(access_token_stores)
    def test_fetch_by_refresh_token_data_not_found(self, store_class):
        cursor_mock = self._cursor_mock()
        cursor_mock.fetchone.return_value = None

        connection_mock = self._con_mock()
        connection_mock.cursor.return_value = cursor_mock

        with self.assertRaises(AccessTokenNotFound):
            store = store_class(connection_mock)
            store.fetch_by_refresh_token("def")

    @with_classes(access_token_stores)
    def test_fetch_existing_token_of_user(self, store_class):
        access_token_row_id = 1
        token_data = {"client_id": "abc", "grant_type": "test_grant",
                      "token": "abc123", "data": {"test": "data"},
                      "expires_at": 1000, "refresh_token": "xyz789",
                      "refresh_expires_at": 2000, "scopes": ["foo", "bar"],
                      "user_id": 1}

        data_cursor = self._cursor_mock()
        data_cursor.fetchall.return_value = [("test", "data")]

        scope_cursor = self._cursor_mock()
        scope_cursor.fetchall.return_value = [("foo",), ("bar",)]

        token_cursor = self._cursor_mock()
        token_cursor.fetchone.return_value = (access_token_row_id,
                                              token_data["client_id"],
                                              token_data["grant_type"],
                                              token_data["token"],
                                              token_data["expires_at"],
                                              token_data["refresh_token"],
                                              token_data["refresh_expires_at"],
                                              token_data["user_id"])

        connection_mock = self._con_mock([token_cursor, scope_cursor,
                                          data_cursor])

        store = store_class(connection=connection_mock)
        access_token = store.\
            fetch_existing_token_of_user(client_id=token_data["client_id"],
                                         grant_type=token_data["grant_type"],
                                         user_id=token_data["user_id"])

        self.assertTrue(isinstance(access_token, AccessToken))
        self.assertEqual(access_token.client_id, token_data["client_id"])
        self.assertEqual(access_token.grant_type, token_data["grant_type"])
        self.assertEqual(access_token.token, token_data["token"])
        self.assertDictEqual(access_token.data, token_data["data"])
        self.assertEqual(access_token.expires_at, token_data["expires_at"])
        self.assertEqual(access_token.refresh_token,
                         token_data["refresh_token"])
        self.assertEqual(access_token.refresh_expires_at,
                         token_data["refresh_expires_at"])
        self.assertListEqual(access_token.scopes, token_data["scopes"])
        self.assertEqual(access_token.user_id, token_data["user_id"])

        token_cursor.execute.\
            assert_called_with(store_class.fetch_existing_token_of_user_query,
                               (token_data["client_id"],
                                token_data["grant_type"],
                                token_data["user_id"]))

        scope_cursor.execute.\
            assert_called_with(store_class.fetch_scopes_by_access_token_query,
                               (access_token_row_id,))

        data_cursor.execute.\
            assert_called_with(store_class.fetch_data_by_access_token_query,
                               (access_token_row_id,))

    @with_classes(access_token_stores)
    def test_fetch_existing_token_of_user_no_token_found(self, store_class):
        cursor_mock = self._cursor_mock()
        cursor_mock.fetchone.return_value = None

        connection_mock = self._con_mock()
        connection_mock.cursor.return_value = cursor_mock

        store = store_class(connection=connection_mock)

        with self.assertRaises(AccessTokenNotFound):
            store.fetch_existing_token_of_user(client_id="abc",
                                               grant_type="test_grant",
                                               user_id=1)

    @with_classes(access_token_stores)
    def test_save_token(self, store_class):
        first_cursor = Mock(spec=["close", "execute"])
        first_cursor.lastrowid = 1

        second_cursor = Mock()
        third_cursor = Mock()
        fourth_cursor = Mock()

        connection_mock = Mock(spec=["commit", "cursor"])
        connection_mock.cursor.side_effect = [first_cursor, second_cursor,
                                              third_cursor, fourth_cursor]

        access_token = AccessToken(client_id="abc", grant_type="test",
                                   token="abc123", data={"test": "data"},
                                   expires_at=1000, refresh_token="xyz789",
                                   refresh_expires_at=2000,
                                   scopes=["foo", "bar"], user_id=1)

        store = store_class(connection=connection_mock)
        result = store.save_token(access_token)

        self.assertTrue(result)

        self.assertEqual(connection_mock.commit.call_count, 4)

        first_cursor.execute.\
            assert_called_with(store_class.create_access_token_query,
                               (access_token.client_id,
                                access_token.grant_type, access_token.token,
                                access_token.expires_at,
                                access_token.refresh_token,
                                access_token.refresh_expires_at,
                                access_token.user_id))
        first_cursor.close.assert_called_with()

        second_cursor.execute.\
            assert_called_with(store_class.create_data_query,
                               ("test", "data", 1))
        second_cursor.close.assert_called_with()

        third_cursor.execute.\
            assert_called_with(store_class.create_scope_query, ("foo", 1))
        third_cursor.close.assert_called_with()

        fourth_cursor.execute.\
            assert_called_with(store_class.create_scope_query, ("bar", 1))
        fourth_cursor.close.assert_called_with()


class AuthCodeStoreTestCase(StoreTestCase):
    @with_classes(auth_code_stores)
    def test_delete_code(self, store_class):
        code = "abc123"

        cursor_mock = self._cursor_mock()
        cursor_mock.lastrowid = None

        connection_mock = self._con_mock([cursor_mock])

        store = store_class(connection=connection_mock)
        store.delete_code(code)

        cursor_mock.execute.\
            assert_called_with(store_class.delete_code_query, (code,))
        self.assertEqual(cursor_mock.close.call_count, 1)

    @with_classes(auth_code_stores)
    def test_fetch_by_code(self, store_class):
        code_data = {"client_id": "abc", "code": "abc123", "expires_at": 1000,
                     "redirect_uri": "http://localhost",
                     "scopes": ["foo", "bar"], "data": {"test": "data"},
                     "user_id": 1}
        id_in_db = 1

        code_cursor = Mock(spec=["execute", "close", "fetchone"])
        code_cursor.lastrowid = 1
        code_cursor.fetchone.return_value = (id_in_db, code_data["client_id"],
                                             code_data["code"],
                                             code_data["expires_at"],
                                             code_data["redirect_uri"],
                                             code_data["user_id"])

        data_cursor = Mock(spec=["execute", "close", "fetchall"])
        data_cursor.fetchall.return_value = [("test", "data")]

        scope_cursor = Mock(spec=["execute", "close", "fetchall"])
        scope_cursor.fetchall.return_value = [("foo",), ("bar",)]

        connection_mock = Mock(spec=["commit", "cursor"])
        connection_mock.cursor.side_effect = [code_cursor, data_cursor,
                                              scope_cursor]

        store = store_class(connection=connection_mock)
        auth_code = store.fetch_by_code("abc123")

        self.assertTrue(isinstance(auth_code, AuthorizationCode))

        self.assertEqual(auth_code.client_id, code_data["client_id"])
        self.assertEqual(auth_code.code, code_data["code"])
        self.assertEqual(auth_code.expires_at, code_data["expires_at"])
        self.assertEqual(auth_code.redirect_uri, code_data["redirect_uri"])
        self.assertEqual(auth_code.user_id, code_data["user_id"])
        self.assertListEqual(auth_code.scopes, code_data["scopes"])
        self.assertDictEqual(auth_code.data, code_data["data"])

        code_cursor.execute.assert_called_with(store_class.fetch_code_query,
                                               (code_data["code"],))
        code_cursor.close.assert_called_with()

        data_cursor.execute.assert_called_with(store_class.fetch_data_query,
                                               (id_in_db,))
        data_cursor.close.assert_called_with()

        scope_cursor.execute.\
            assert_called_with(store_class.fetch_scopes_query, (id_in_db,))

    @with_classes(auth_code_stores)
    def test_fetch_by_code_no_data_found(self, store_class):
        cursor_mock = Mock(spec=["execute", "close", "fetchone"])
        cursor_mock.fetchone.return_value = None

        connection_mock = Mock(spec=["cursor"])
        connection_mock.cursor.return_value = cursor_mock

        with self.assertRaises(AuthCodeNotFound):
            store = store_class(connection_mock)
            store.fetch_by_code("xyz789")

    @with_classes(auth_code_stores)
    def test_save_code(self, store_class):
        code_data = {"client_id": "abc", "code": "abc123", "expires_at": 1000,
                     "redirect_uri": "http://localhost",
                     "scopes": ["foo", "bar"], "data": {"test": "data"},
                     "user_id": 1}
        id_in_db = 1

        create_code_cursor = Mock(spec=["close", "execute"])
        create_code_cursor.lastrowid = id_in_db
        create_data_cursor = Mock(spec=["close", "execute"])
        create_data_cursor.lastrowid = 1
        create_first_scope_cursor = Mock(spec=["close", "execute"])
        create_first_scope_cursor.lastrowid = 1
        create_second_scope_cursor = Mock(spec=["close", "execute"])
        create_second_scope_cursor.lastrowid = 2

        connection_mock = Mock(spec=["commit", "cursor"])
        connection_mock.cursor.side_effect = [create_code_cursor,
                                              create_data_cursor,
                                              create_first_scope_cursor,
                                              create_second_scope_cursor]

        auth_code = AuthorizationCode(**code_data)

        store = store_class(connection=connection_mock)
        result = store.save_code(auth_code)

        self.assertTrue(result)

        create_code_cursor.execute.\
            assert_called_with(store_class.create_auth_code_query,
                               (code_data["client_id"], code_data["code"],
                                code_data["expires_at"],
                                code_data["redirect_uri"],
                                code_data["user_id"]))
        create_code_cursor.close.assert_called_with()

        create_data_cursor.execute.\
            assert_called_with(store_class.create_data_query,
                               ("test", "data", id_in_db))
        create_data_cursor.close.assert_called_with()

        create_first_scope_cursor.execute.\
            assert_called_with(store_class.create_scope_query,
                               ("foo", id_in_db))

        create_second_scope_cursor.execute.\
            assert_called_with(store_class.create_scope_query,
                               ("bar", id_in_db))

        self.assertEqual(connection_mock.commit.call_count, 4)


class MysqlClientStoreTestCase(StoreTestCase):
    @with_classes(client_stores)
    def test_fetch_by_client_id(self, store_class):
        client_id = 123
        client_data = {"identifier": "abc", "secret": "xyz",
                       "authorized_grants": ["authorization_code",
                                             "password"],
                       "authorized_response_types": ["code", "token"],
                       "redirect_uris": ["http://example.com",
                                         "http://www.example.com"]}

        client_cursor = Mock(spec=["close", "execute", "fetchone"])
        client_cursor.fetchone.return_value = (client_id,
                                               client_data["identifier"],
                                               client_data["secret"])
        redirect_uris_cursor = Mock(spec=["close", "execute", "fetchall"])
        redirect_uris_cursor.fetchall.return_value = [
            ("http://example.com",), ("http://www.example.com",)
        ]
        grants_cursor = Mock(spec=["close", "execute", "fetchall"])
        grants_cursor.fetchall.return_value = [("authorization_code",),
                                               ("password",)]
        response_types_cursor = Mock(spec=["close", "execute", "fetchall"])
        response_types_cursor.fetchall.return_value = [("code",), ("token",)]

        connection_mock = Mock(spec=["commit", "cursor"])

        connection_mock.cursor.side_effect = [client_cursor,
                                              grants_cursor,
                                              redirect_uris_cursor,
                                              response_types_cursor]

        store = store_class(connection=connection_mock)
        client = store.fetch_by_client_id(client_data["identifier"])

        self.assertTrue(isinstance(client, Client))
        self.assertEqual(client.identifier, client_data["identifier"])
        self.assertEqual(client.secret, client_data["secret"])
        self.assertListEqual(client.authorized_grants,
                             client_data["authorized_grants"])
        self.assertListEqual(client.authorized_response_types,
                             client_data["authorized_response_types"])
        self.assertListEqual(client.redirect_uris,
                             client_data["redirect_uris"])

        client_cursor.execute.\
            assert_called_with(store_class.fetch_client_query,
                               (client_data["identifier"],))
        redirect_uris_cursor.execute.\
            assert_called_with(store_class.fetch_redirect_uris_query,
                               (client_id,))
        grants_cursor.execute.\
            assert_called_with(store_class.fetch_grants_query, (client_id,))
        response_types_cursor.execute.\
            assert_called_with(store_class.fetch_response_types_query,
                               (client_id,))

    @with_classes(client_stores)
    def test_fetch_by_client_id_client_not_found(self, store_class):
        client_cursor_mock = Mock(spec=["close", "execute", "fetchone"])
        client_cursor_mock.fetchone.return_value = None

        connection_mock = Mock(spec=["cursor"])
        connection_mock.cursor.return_value = client_cursor_mock

        store = store_class(connection=connection_mock)

        with self.assertRaises(ClientNotFoundError):
            store.fetch_by_client_id("abc")



import sys

# Enables unit tests to work under Python 2.6
# Code copied from
# https://github.com/facebook/tornado/blob/master/tornado/test/util.py
if sys.version_info >= (2, 7):
    import unittest
else:
    import unittest2 as unittest



# -*- coding: utf-8 -*-
import memcache

from oauth2.datatype import AccessToken, AuthorizationCode
from oauth2.error import AccessTokenNotFound, AuthCodeNotFound
from oauth2.store import AccessTokenStore, AuthCodeStore


class TokenStore(AccessTokenStore, AuthCodeStore):
    """
    Uses memcache to store access tokens and auth tokens.

    This Store supports ``python-memcached``. Arguments are passed to the
    underlying client implementation.

    Initialization by passing an object::

        # This example uses python-memcached
        import memcache

        # Somewhere in your application
        mc = memcache.Client(servers=['127.0.0.1:11211'], debug=0)
        # ...
        token_store = TokenStore(mc=mc)

    Initialization using ``python-memcached``::

        token_store = TokenStore(servers=['127.0.0.1:11211'], debug=0)

    """
    def __init__(self, mc=None, prefix="oauth2", *args, **kwargs):
        self.prefix = prefix

        if mc is not None:
            self.mc = mc
        else:
            self.mc = memcache.Client(*args, **kwargs)

    def fetch_by_code(self, code):
        """
        Returns data belonging to an authorization code from memcache or
        ``None`` if no data was found.

        See :class:`oauth2.store.AuthCodeStore`.

        """
        code_data = self.mc.get(self._generate_cache_key(code))

        if code_data is None:
            raise AuthCodeNotFound

        return AuthorizationCode(**code_data)

    def save_code(self, authorization_code):
        """
        Stores the data belonging to an authorization code token in memcache.

        See :class:`oauth2.store.AuthCodeStore`.

        """
        key = self._generate_cache_key(authorization_code.code)

        self.mc.set(key, {"client_id": authorization_code.client_id,
                          "code": authorization_code.code,
                          "expires_at": authorization_code.expires_at,
                          "redirect_uri": authorization_code.redirect_uri,
                          "scopes": authorization_code.scopes,
                          "data": authorization_code.data,
                          "user_id": authorization_code.user_id})

    def delete_code(self, code):
        """
        Deletes an authorization code after use
        :param code: The authorization code.
        """
        self.mc.delete(self._generate_cache_key(code))

    def save_token(self, access_token):
        """
        Stores the access token and additional data in memcache.

        See :class:`oauth2.store.AccessTokenStore`.

        """
        key = self._generate_cache_key(access_token.token)
        self.mc.set(key, access_token.__dict__)

        unique_token_key = self._unique_token_key(access_token.client_id,
                                                  access_token.grant_type,
                                                  access_token.user_id)
        self.mc.set(self._generate_cache_key(unique_token_key),
                    access_token.__dict__)

        if access_token.refresh_token is not None:
            rft_key = self._generate_cache_key(access_token.refresh_token)
            self.mc.set(rft_key, access_token.__dict__)

    def delete_refresh_token(self, refresh_token):
        """
        Deletes a refresh token after use
        :param refresh_token: The refresh token to delete.
        """
        access_token = self.fetch_by_refresh_token(refresh_token)
        self.mc.delete(self._generate_cache_key(access_token.token))
        self.mc.delete(self._generate_cache_key(refresh_token))

    def fetch_by_refresh_token(self, refresh_token):
        token_data = self.mc.get(refresh_token)

        if token_data is None:
            raise AccessTokenNotFound

        return AccessToken(**token_data)

    def fetch_existing_token_of_user(self, client_id, grant_type, user_id):
        data = self.mc.get(self._unique_token_key(client_id, grant_type,
                                                  user_id))

        if data is None:
            raise AccessTokenNotFound

        return AccessToken(**data)

    def _unique_token_key(self, client_id, grant_type, user_id):
        return "{0}_{1}_{2}".format(client_id, grant_type, user_id)

    def _generate_cache_key(self, identifier):
        return self.prefix + "_" + identifier



"""
Read or write data from or to local memory.

Though not very valuable in a production setup, these store adapters are great
for testing purposes.
"""

from oauth2.store import AccessTokenStore, AuthCodeStore, ClientStore
from oauth2.error import AccessTokenNotFound, AuthCodeNotFound, \
                         ClientNotFoundError
from oauth2.datatype import Client


class ClientStore(ClientStore):
    """
    Stores clients in memory.
    """
    def __init__(self):
        self.clients = {}

    def add_client(self, client_id, client_secret, redirect_uris,
                   authorized_grants=None, authorized_response_types=None):
        """
        Add a client app.

        :param client_id: Identifier of the client app.
        :param client_secret: Secret the client app uses for authentication
                              against the OAuth 2.0 provider.
        :param redirect_uris: A ``list`` of URIs to redirect to.

        """
        self.clients[client_id] = Client(
            identifier=client_id,
            secret=client_secret,
            redirect_uris=redirect_uris,
            authorized_grants=authorized_grants,
            authorized_response_types=authorized_response_types)

        return True

    def fetch_by_client_id(self, client_id):
        """
        Retrieve a client by its identifier.

        :param client_id: Identifier of a client app.
        :return: An instance of :class:`oauth2.Client`.
        :raises: ClientNotFoundError

        """
        if client_id not in self.clients:
            raise ClientNotFoundError

        return self.clients[client_id]


class TokenStore(AccessTokenStore, AuthCodeStore):
    """
    Stores tokens in memory.

    Useful for testing purposes or APIs with a very limited set of clients.
    Use memcache or redis as storage to be able to scale.
    """
    def __init__(self):
        self.access_tokens = {}
        self.auth_codes = {}
        self.refresh_tokens = {}
        self.unique_token_identifier = {}

    def fetch_by_code(self, code):
        """
        Returns an AuthorizationCode.

        :param code: The authorization code.
        :return: An instance of :class:`oauth2.datatype.AuthorizationCode`.
        :raises: :class:`AuthCodeNotFound` if no data could be retrieved for
                 given code.

        """
        if code not in self.auth_codes:
            raise AuthCodeNotFound

        return self.auth_codes[code]

    def save_code(self, authorization_code):
        """
        Stores the data belonging to an authorization code token.

        :param authorization_code: An instance of
                                   :class:`oauth2.datatype.AuthorizationCode`.

        """
        self.auth_codes[authorization_code.code] = authorization_code

        return True

    def save_token(self, access_token):
        """
        Stores an access token and additional data in memory.

        :param access_token: An instance of :class:`oauth2.datatype.AccessToken`.
        """
        self.access_tokens[access_token.token] = access_token

        unique_token_key = self._unique_token_key(access_token.client_id,
                                                  access_token.grant_type,
                                                  access_token.user_id)

        self.unique_token_identifier[unique_token_key] = access_token.token

        if access_token.refresh_token is not None:
            self.refresh_tokens[access_token.refresh_token] = access_token

        return True

    def delete_code(self, code):
        """
        Deletes an authorization code after use
        :param code: The authorization code.
        """
        if code in self.auth_codes:
            del self.auth_codes[code]

    def delete_refresh_token(self, refresh_token):
        """
        Deletes a refresh token after use
        :param refresh_token: The refresh_token.
        """
        if refresh_token in self.refresh_tokens:
            del self.refresh_tokens[refresh_token]

    def fetch_by_refresh_token(self, refresh_token):
        """
        Find an access token by its refresh token.

        :param refresh_token: The refresh token that was assigned to an
                              ``AccessToken``.
        :return: The :class:`oauth2.datatype.AccessToken`.
        :raises: :class:`oauth2.error.AccessTokenNotFound`
        """
        if refresh_token not in self.refresh_tokens:
            raise AccessTokenNotFound

        return self.refresh_tokens[refresh_token]

    def fetch_by_token(self, token):
        """
        Returns data associated with an access token or ``None`` if no data
        was found.

        Useful for cases like validation where the access token needs to be
        read again.

        :param token: A access token code.
        :return: An instance of :class:`oauth2.datatype.AccessToken`.
        """
        if token not in self.access_tokens:
            raise AccessTokenNotFound

        return self.access_tokens[token]

    def fetch_existing_token_of_user(self, client_id, grant_type, user_id):
        try:
            key = self._unique_token_key(client_id, grant_type, user_id)
            token = self.unique_token_identifier[key]
        except KeyError:
            raise AccessTokenNotFound

        return self.fetch_by_token(token)

    def _unique_token_key(self, client_id, grant_type, user_id):
        return "{0}_{1}_{2}".format(client_id, grant_type, user_id)



from __future__ import absolute_import

import json
from multiprocessing import Queue
from wsgiref.simple_server import make_server
from oauth2 import Provider
from oauth2.grant import AuthorizationCodeGrant, RefreshToken
from oauth2.test import unittest
from oauth2.test.functional import NoLoggingHandler
from oauth2.tokengenerator import Uuid4
from oauth2.web import AuthorizationCodeGrantSiteAdapter
from oauth2.web.wsgi import Application
from ..functional import store_factory
from tornado.ioloop import IOLoop
from tornado.web import Application as TornadoApplication
from tornado.web import url
from ...web.tornado import OAuth2Handler

try:
    from urllib.request import urlopen
    from urllib.parse import parse_qs, urlencode
    from urllib.error import HTTPError
except ImportError:
    from urllib import urlencode
    from urllib2 import urlopen, HTTPError
    from urlparse import parse_qs

try:
    from multiprocessing.process import Process
except ImportError:
    from multiprocessing import Process


def create_provider():
    redirect_uri = "http://127.0.0.1:15487/callback"

    stores = store_factory(client_identifier="abc",
                           client_secret="xyz",
                           redirect_uris=[redirect_uri])

    provider = Provider(access_token_store=stores["access_token_store"],
                        auth_code_store=stores["auth_code_store"],
                        client_store=stores["client_store"],
                        token_generator=Uuid4())

    provider.add_grant(
        AuthorizationCodeGrant(
            expires_in=120,
            site_adapter=TestSiteAdapter()
        )
    )

    provider.add_grant(RefreshToken(expires_in=60))

    return provider


def run_client(queue):
    try:
        app = ClientApplication(
            callback_url="http://127.0.0.1:15487/callback",
            client_id="abc",
            client_secret="xyz",
            provider_url="http://127.0.0.1:15486")

        httpd = make_server('', 15487, app,
                            handler_class=NoLoggingHandler)

        queue.put({"result": 0})

        httpd.serve_forever()
    except Exception as e:
        queue.put({"result": 1, "error_message": str(e)})


class AuthorizationCodeTestCase(unittest.TestCase):
    def setUp(self):
        self.client = None
        self.server = None

    def test_tornado(self):
        def run_provider(queue):
            try:
                provider = create_provider()

                app = TornadoApplication([
                    url(r"/authorize", OAuth2Handler, dict(provider=provider)),
                    url(r"/token", OAuth2Handler, dict(provider=provider))
                ], debug=True)
                app.listen(15486)

                queue.put({"result": 0})

                IOLoop.current().start()
            except Exception as e:
                queue.put({"result": 1, "error_message": str(e)})

        ready_queue = Queue()

        self.server = Process(target=run_provider, args=(ready_queue,))
        self.server.start()

        provider_started = ready_queue.get()

        if provider_started["result"] != 0:
            raise Exception("Error starting Provider process with message"
                            "'{0}'".format(provider_started["error_message"]))

        self.client = Process(target=run_client, args=(ready_queue,))
        self.client.start()

        client_started = ready_queue.get()

        if client_started["result"] != 0:
            raise Exception("Error starting Client Application process with "
                            "message '{0}'"
                            .format(client_started["error_message"]))

        self.access_token()

    def test_wsgi(self):
        def run_provider(queue):
            try:
                provider = create_provider()

                app = Application(provider=provider)

                httpd = make_server('', 15486, app,
                                    handler_class=NoLoggingHandler)

                queue.put({"result": 0})

                httpd.serve_forever()
            except Exception as e:
                queue.put({"result": 1, "error_message": str(e)})

        ready_queue = Queue()

        self.server = Process(target=run_provider, args=(ready_queue,))
        self.server.start()

        provider_started = ready_queue.get()

        if provider_started["result"] != 0:
            raise Exception("Error starting Provider process with message"
                            "'{0}'".format(provider_started["error_message"]))

        self.client = Process(target=run_client, args=(ready_queue,))
        self.client.start()

        client_started = ready_queue.get()

        if client_started["result"] != 0:
            raise Exception("Error starting Client Application process with "
                            "message '{0}'"
                            .format(client_started["error_message"]))

        self.access_token()

    def test_wsgi_404(self):
        def run_provider(queue):
            try:
                provider = create_provider()

                app = Application(provider=provider)

                httpd = make_server('', 15486, app,
                                    handler_class=NoLoggingHandler)

                queue.put({"result": 0})

                httpd.serve_forever()
            except Exception as e:
                queue.put({"result": 1, "error_message": str(e)})

        ready_queue = Queue()

        self.server = Process(target=run_provider, args=(ready_queue,))
        self.server.start()

        provider_started = ready_queue.get()

        if provider_started["result"] != 0:
            raise Exception("Error starting Provider process with message"
                            "'{0}'".format(provider_started["error_message"]))

        try:
            urlopen("http://127.0.0.1:15486/invalid-path").read()
        except HTTPError as e:
            self.assertEqual(404, e.code)

    def access_token(self):
        uuid_regex = "^[a-z0-9]{8}\-[a-z0-9]{4}\-[a-z0-9]{4}\-[a-z0-9]{4}-[a-z0-9]{12}$"

        try:
            access_token_result = urlopen("http://127.0.0.1:15487/app").read()
        except HTTPError as e:
            print(e.read())
            exit(1)

        access_token_data = json.loads(access_token_result.decode('utf-8'))

        self.assertEqual(access_token_data["token_type"], "Bearer")
        self.assertEqual(access_token_data["expires_in"], 120)
        self.assertRegexpMatches(access_token_data["access_token"],
                                 uuid_regex)
        self.assertRegexpMatches(access_token_data["refresh_token"],
                                 uuid_regex)

        request_data = {"grant_type": "refresh_token",
                        "refresh_token": access_token_data["refresh_token"],
                        "client_id": "abc",
                        "client_secret": "xyz"}

        refresh_token_result = urlopen(
            "http://127.0.0.1:15486/token",
            urlencode(request_data).encode('utf-8')
        )

        refresh_token_data = json.loads(refresh_token_result.read().decode('utf-8'))

        self.assertEqual(refresh_token_data["token_type"], "Bearer")
        self.assertEqual(refresh_token_data["expires_in"], 120)
        self.assertRegexpMatches(refresh_token_data["access_token"],
                                 uuid_regex)

    def tearDown(self):
        if self.client is not None:
            self.client.terminate()
            self.client.join()

        if self.server is not None:
            self.server.terminate()
            self.server.join()


class TestSiteAdapter(AuthorizationCodeGrantSiteAdapter):
    def authenticate(self, request, environ, scopes, client):
        return {"additional": "data"}, 1

    def user_has_denied_access(self, request):
        return False


class ClientApplication(object):
    def __init__(self, callback_url, client_id, client_secret, provider_url):
        self.callback_url = callback_url
        self.client_id = client_id
        self.client_secret = client_secret
        self.api_server_url = provider_url

        self.access_token_result = None
        self.auth_token = None
        self.token_type = ""

    def __call__(self, env, start_response):
        try:
            if env["PATH_INFO"] == "/app":
                status, body, headers = self._serve_application()
            elif env["PATH_INFO"] == "/callback":
                status, body, headers = self._read_auth_token(env)
            else:
                status = "301 Moved"
                body = ""
                headers = {"Location": "/app"}
        except HTTPError as http_error:
            print("HTTPError occured:")
            print(http_error.read())
            raise

        start_response(status,
                       [(header, val) for header, val in list(headers.items())])
        return [body]

    def _request_access_token(self):
        post_params = {"client_id": self.client_id,
                       "client_secret": self.client_secret,
                       "code": self.auth_token,
                       "grant_type": "authorization_code",
                       "redirect_uri": self.callback_url}
        token_endpoint = self.api_server_url + "/token"

        result = urlopen(token_endpoint,
                         urlencode(post_params).encode('utf-8'))

        result = json.loads(result.read().decode('utf-8'))
        self.access_token_result = result

        return "302 Found", b"", {"Location": "/app"}

    def _read_auth_token(self, env):
        query_params = parse_qs(env["QUERY_STRING"])
        self.auth_token = query_params["code"][0]

        return "302 Found", b"", {"Location": "/app"}

    def _request_auth_token(self):
        auth_endpoint = self.api_server_url + "/authorize"
        query = urlencode({"client_id": "abc",
                           "redirect_uri": self.callback_url,
                           "response_type": "code"})

        location = "%s?%s" % (auth_endpoint, query)

        return "302 Found", b"", {"Location": location}

    def _serve_application(self):
        if self.access_token_result is None:
            if self.auth_token is None:
                return self._request_auth_token()
            else:
                return self._request_access_token()
        else:
            return ("200 OK",
                    json.dumps(self.access_token_result).encode('utf-8'), {})



"""
Store adapters to persist and retrieve data during the OAuth 2.0 process or
for later use.
This module provides base classes that can be extended to implement your own
solution specific to your needs.
It also includes implementations for popular storage systems like memcache.
"""


class AccessTokenStore(object):
    """
    Base class for persisting an access token after it has been generated.

    Used by two-legged and three-legged authentication flows.
    """
    def save_token(self, access_token):
        """
        Stores an access token and additional data.

        :param access_token: An instance of :class:`oauth2.datatype.AccessToken`.

        """
        raise NotImplementedError

    def fetch_existing_token_of_user(self, client_id, grant_type, user_id):
        """
        Fetches an access token identified by its client id, type of grant and
        user id.

        This method must be implemented to make use of unique access tokens.

        :param client_id: Identifier of the client a token belongs to.
        :param grant_type: The type of the grant that created the token
        :param user_id: Identifier of the user a token belongs to.
        :return: An instance of :class:`oauth2.datatype.AccessToken`.
        :raises: :class:`oauth2.error.AccessTokenNotFound` if no data could be
                 retrieved.
        """
        raise NotImplementedError

    def fetch_by_refresh_token(self, refresh_token):
        """
        Fetches an access token from the store using its refresh token to
        identify it.

        :param refresh_token: A string containing the refresh token.
        :return: An instance of :class:`oauth2.datatype.AccessToken`.
        :raises: :class:`oauth2.error.AccessTokenNotFound` if no data could be retrieved for
                 given refresh_token.
        """
        raise NotImplementedError

    def delete_refresh_token(self, refresh_token):
        """
        Deletes an access token from the store using its refresh token to identify it.
        This invalidates both the access token and the refresh token.

        :param refresh_token: A string containing the refresh token.
        :return: None.
        :raises: :class:`oauth2.error.AccessTokenNotFound` if no data could be retrieved for
                 given refresh_token.
        """
        raise NotImplementedError


class AuthCodeStore(object):
    """
    Base class for persisting and retrieving an auth token during the
    Authorization Code Grant flow.
    """
    def fetch_by_code(self, code):
        """
        Returns an AuthorizationCode fetched from a storage.

        :param code: The authorization code.
        :return: An instance of :class:`oauth2.datatype.AuthorizationCode`.
        :raises: :class:`oauth2.error.AuthCodeNotFound` if no data could be retrieved for
                 given code.

        """
        raise NotImplementedError

    def save_code(self, authorization_code):
        """
        Stores the data belonging to an authorization code token.

        :param authorization_code: An instance of
                                   :class:`oauth2.datatype.AuthorizationCode`.
        """
        raise NotImplementedError

    def delete_code(self, code):
        """
        Deletes an authorization code after it's use per section 4.1.2.

        http://tools.ietf.org/html/rfc6749#section-4.1.2

        :param code: The authorization code.
        """
        raise NotImplementedError


class ClientStore(object):
    """
    Base class for handling OAuth2 clients.
    """
    def fetch_by_client_id(self, client_id):
        """
        Retrieve a client by its identifier.

        :param client_id: Identifier of a client app.
        :return: An instance of :class:`oauth2.datatype.Client`.
        :raises: :class:`oauth2.error.ClientNotFoundError` if no data could be retrieved for
                 given client_id.
        """
        raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class AuthenticatingSiteAdapter(object):
    """
    Extended by site adapters that need to authenticate the user.
    """
    def authenticate(self, request, environ, scopes, client):
        """
        Authenticates a user and checks if she has authorized access.

        :param request: Incoming request data.
        :type request: oauth2.web.Request

        :param environ: Environment variables of the request.
        :type environ: dict

        :param scopes: A list of strings with each string being one requested
                       scope.
        :type scopes: list

        :param client: The client that initiated the authorization process
        :type client: oauth2.datatype.Client

        :return: A ``dict`` containing arbitrary data that will be passed to
                 the current storage adapter and saved with auth code and
                 access token. Return a tuple in the form
                 `(additional_data, user_id)` if you want to use
                 :doc:`unique_token`.
        :rtype: dict

        :raises oauth2.error.UserNotAuthenticated: If the user could not be
                                                   authenticated.
        """
        raise NotImplementedError


class UserFacingSiteAdapter(object):
    """
    Extended by site adapters that need to interact with the user.

    Display HTML or redirect the user agent to another page of your website
    where she can do something before being returned to the OAuth 2.0 server.
    """
    def render_auth_page(self, request, response, environ, scopes, client):
        """
        Defines how to display a confirmation page to the user.

        :param request: Incoming request data.
        :type request: oauth2.web.Request

        :param response: Response to return to a client.
        :type response: oauth2.web.Response

        :param environ: Environment variables of the request.
        :type environ: dict

        :param scopes: A list of strings with each string being one requested
                       scope.
        :type scopes: list

        :param client: The client that initiated the authorization process
        :type client: oauth2.datatype.Client

        :return: The response passed in as a parameter.
                 It can contain HTML or issue a redirect.
        :rtype: oauth2.web.Response
        """
        raise NotImplementedError

    def user_has_denied_access(self, request):
        """
        Checks if the user has denied access. This will lead to python-oauth2
        returning a "acess_denied" response to the requesting client app.

        :param request: Incoming request data.
        :type request: oauth2.web.Request

        :return: Return ``True`` if the user has denied access.
        :rtype: bool
        """
        raise NotImplementedError


class AuthorizationCodeGrantSiteAdapter(UserFacingSiteAdapter,
                                        AuthenticatingSiteAdapter):
    """
    Definition of a site adapter as required by
    :class:`oauth2.grant.AuthorizationCodeGrant`.
    """
    pass


class ImplicitGrantSiteAdapter(UserFacingSiteAdapter,
                               AuthenticatingSiteAdapter):
    """
    Definition of a site adapter as required by
    :class:`oauth2.grant.ImplicitGrant`.
    """
    pass


class ResourceOwnerGrantSiteAdapter(AuthenticatingSiteAdapter):
    """
    Definition of a site adapter as required by
    :class:`oauth2.grant.ResourceOwnerGrant`.
    """
    pass


class Request(object):
    """
    Base class defining the interface of a request.
    """
    @property
    def method(self):
        """
        Returns the HTTP method of the request.
        """
        raise NotImplementedError

    @property
    def path(self):
        """
        Returns the current path portion of the current uri.

        Used by some grants to determine which action to take.
        """
        raise NotImplementedError

    def get_param(self, name, default=None):
        """
        Retrieve a parameter from the query string of the request.
        """
        raise NotImplementedError

    def header(self, name, default=None):
        """
        Retrieve a header of the request.
        """
        raise NotImplementedError

    def post_param(self, name, default=None):
        """
        Retrieve a parameter from the body of the request.
        """
        raise NotImplementedError


class Response(object):
    """
    Contains data returned to the requesting user agent.
    """
    def __init__(self):
        self.status_code = 200
        self._headers = {"Content-Type": "text/html"}
        self.body = ""

    @property
    def headers(self):
        return self._headers

    def add_header(self, header, value):
        """
        Add a header to the response.
        """
        self._headers[header] = str(value)



"""
Errors raised during the OAuth 2.0 flow.
"""


class AccessTokenNotFound(Exception):
    """
    Error indicating that an access token could not be read from the
    storage backend by an instance of :class:`oauth2.store.AccessTokenStore`.
    """
    pass


class AuthCodeNotFound(Exception):
    """
    Error indicating that an authorization code could not be read from the
    storage backend by an instance of :class:`oauth2.store.AuthCodeStore`.
    """
    pass


class ClientNotFoundError(Exception):
    """
    Error raised by an implementation of :class:`oauth2.store.ClientStore` if
    a client does not exists.
    """
    pass


class InvalidSiteAdapter(Exception):
    """
    Raised by :class:`oauth2.grant.SiteAdapterMixin` in case an invalid site
    adapter was passed to the instance.
    """
    pass


class UserIdentifierMissingError(Exception):
    """
    Indicates that the identifier of a user is missing when the use of unique
    access token is enabled.
    """
    pass


class OAuthBaseError(Exception):
    """
    Base class used by all OAuth 2.0 errors.

    :param error: Identifier of the error.
    :param error_uri: Set this to delivery an URL to your documentation that
                      describes the error. (optional)
    :param explanation: Short message that describes the error. (optional)
    """
    def __init__(self, error, error_uri=None, explanation=None):
        self.error = error
        self.error_uri = error_uri
        self.explanation = explanation

        super(OAuthBaseError, self).__init__()


class OAuthInvalidError(OAuthBaseError):
    """
    Indicates an error during validation of a request.
    """
    pass


class OAuthInvalidNoRedirectError(OAuthInvalidError):
    """
    Indicates an error during validation of a request.
    The provider will not inform the client about the error by redirecting to
    it. This behaviour is required by the Authorization Request step of the
    Authorization Code Grant and Implicit Grant.
    """
    pass


class UnsupportedGrantError(Exception):
    """
    Indicates that a requested grant is not supported by the server.
    """
    pass


class RedirectUriUnknown(Exception):
    """
    Indicates that a redirect_uri is not associated with a client.
    """
    pass


class UserNotAuthenticated(Exception):
    """
    Raised by a :class:`oauth2.web.SiteAdapter` if a user could not be
    authenticated.
    """
    pass



import base64
from base64 import b64encode
from oauth2.test import unittest
from mock import Mock
from oauth2.client_authenticator import ClientAuthenticator, http_basic_auth, \
    request_body
from oauth2.datatype import Client
from oauth2.error import OAuthInvalidNoRedirectError, ClientNotFoundError,\
    OAuthInvalidError
from oauth2.store import ClientStore
from oauth2.web.wsgi import Request


class ClientAuthenticatorTestCase(unittest.TestCase):
    def setUp(self):
        self.client = Client(identifier="abc", secret="xyz",
                             authorized_grants=["authorization_code"],
                             authorized_response_types=["code"],
                             redirect_uris=["http://callback"])
        self.client_store_mock = Mock(spec=ClientStore)

        self.source_mock = Mock()

        self.authenticator = ClientAuthenticator(
            client_store=self.client_store_mock,
            source=self.source_mock)

    def test_by_identifier(self):
        redirect_uri = "http://callback"

        self.client_store_mock.fetch_by_client_id.return_value = self.client

        request_mock = Mock(spec=Request)
        request_mock.get_param.side_effect = [self.client.identifier,
                                              redirect_uri]

        client = self.authenticator.by_identifier(request=request_mock)

        self.client_store_mock.fetch_by_client_id.\
            assert_called_with(self.client.identifier)
        self.assertEqual(client.redirect_uri, redirect_uri)

    def test_by_identifier_client_id_not_set(self):
        request_mock = Mock(spec=Request)
        request_mock.get_param.return_value = None

        with self.assertRaises(OAuthInvalidNoRedirectError) as expected:
            self.authenticator.by_identifier(request=request_mock)

        self.assertEqual(expected.exception.error, "missing_client_id")

    def test_by_identifier_unknown_client(self):
        request_mock = Mock(spec=Request)
        request_mock.get_param.return_value = "def"

        self.client_store_mock.fetch_by_client_id.\
            side_effect = ClientNotFoundError

        with self.assertRaises(OAuthInvalidNoRedirectError) as expected:
            self.authenticator.by_identifier(request=request_mock)

        self.assertEqual(expected.exception.error, "unknown_client")

    def test_by_identifier_unknown_redirect_uri(self):
        response_type = "code"
        unknown_redirect_uri = "http://unknown.com"

        request_mock = Mock(spec=Request)
        request_mock.get_param.side_effect = [self.client.identifier,
                                              response_type,
                                              unknown_redirect_uri]

        self.client_store_mock.fetch_by_client_id.return_value = self.client

        with self.assertRaises(OAuthInvalidNoRedirectError) as expected:
            self.authenticator.by_identifier(request=request_mock)

        self.assertEqual(expected.exception.error, "invalid_redirect_uri")

    def test_by_identifier_secret(self):
        client_id = "abc"
        client_secret = "xyz"
        grant_type = "authorization_code"

        request_mock = Mock(spec=Request)
        request_mock.post_param.return_value = grant_type

        self.source_mock.return_value = (client_id, client_secret)

        self.client_store_mock.fetch_by_client_id.return_value = self.client

        self.authenticator.by_identifier_secret(request=request_mock)
        self.client_store_mock.fetch_by_client_id.\
            assert_called_with(client_id)

    def test_by_identifier_secret_unknown_client(self):
        client_id = "def"
        client_secret = "uvw"

        self.source_mock.return_value = (client_id, client_secret)

        request_mock = Mock(spec=Request)

        self.client_store_mock.fetch_by_client_id.\
            side_effect = ClientNotFoundError

        with self.assertRaises(OAuthInvalidError) as expected:
            self.authenticator.by_identifier_secret(request_mock)

        self.assertEqual(expected.exception.error, "invalid_client")

    def test_by_identifier_secret_client_not_authorized(self):
        client_id = "abc"
        client_secret = "xyz"
        grant_type = "client_credentials"

        self.source_mock.return_value = (client_id, client_secret)

        request_mock = Mock(spec=Request)
        request_mock.post_param.return_value = grant_type

        self.client_store_mock.fetch_by_client_id.return_value = self.client

        with self.assertRaises(OAuthInvalidError) as expected:
            self.authenticator.by_identifier_secret(request_mock)

        self.assertEqual(expected.exception.error, "unauthorized_client")

    def test_by_identifier_secret_wrong_secret(self):
        client_id = "abc"
        client_secret = "uvw"
        grant_type = "authorization_code"

        self.source_mock.return_value = (client_id, client_secret)

        request_mock = Mock(spec=Request)
        request_mock.post_param.return_value = grant_type

        self.client_store_mock.fetch_by_client_id.return_value = self.client

        with self.assertRaises(OAuthInvalidError) as expected:
            self.authenticator.by_identifier_secret(request_mock)

        self.assertEqual(expected.exception.error, "invalid_client")


class RequestBodyTestCase(unittest.TestCase):
    def test_valid(self):
        client_id = "abc"
        client_secret = "secret"

        request_mock = Mock(spec=Request)
        request_mock.post_param.side_effect = [client_id, client_secret]

        result = request_body(request_mock)

        self.assertEqual(result[0], client_id)
        self.assertEqual(result[1], client_secret)

    def test_no_client_id(self):
        request_mock = Mock(spec=Request)
        request_mock.post_param.return_value = None

        with self.assertRaises(OAuthInvalidError) as expected:
            request_body(request_mock)

        self.assertEqual(expected.exception.error, "invalid_request")

    def test_no_client_secret(self):
        request_mock = Mock(spec=Request)
        request_mock.post_param.side_effect = ["abc", None]

        with self.assertRaises(OAuthInvalidError) as expected:
            request_body(request_mock)

        self.assertEqual(expected.exception.error, "invalid_request")


class HttpBasicAuthTestCase(unittest.TestCase):
    def test_valid(self):
        client_id = "testclient"
        client_secret = "secret"

        credentials = "{0}:{1}".format(client_id, client_secret)

        encoded = b64encode(credentials.encode("latin1"))

        request_mock = Mock(spec=Request)
        request_mock.header.return_value = "Basic {0}".\
            format(encoded.decode("latin1"))

        result_client_id, result_client_secret = http_basic_auth(request=request_mock)

        request_mock.header.assert_called_with("authorization")

        self.assertEqual(result_client_id, client_id)
        self.assertEqual(result_client_secret, client_secret)

    def test_header_not_present(self):
        request_mock = Mock(spec=Request)
        request_mock.header.return_value = None

        with self.assertRaises(OAuthInvalidError) as expected:
            http_basic_auth(request=request_mock)

        self.assertEqual(expected.exception.error, "invalid_request")

    def test_invalid_authorization_header(self):
        request_mock = Mock(spec=Request)
        request_mock.header.return_value = "some-data"

        with self.assertRaises(OAuthInvalidError) as expected:
            http_basic_auth(request=request_mock)

        self.assertEqual(expected.exception.error, "invalid_request")



"""
Every client that sends a request to obtain an access token needs to
authenticate with the provider.

The authentication of confidential clients can be handled in several ways,
some of which come bundled with this module.
"""

from base64 import b64decode
from oauth2.error import OAuthInvalidNoRedirectError, RedirectUriUnknown, \
    OAuthInvalidError, ClientNotFoundError


class ClientAuthenticator(object):
    """
    Handles authentication of a client both by its identifier as well as by
    its identifier and secret.

    :param client_store: The Client Store to retrieve a client from.
    :type client_store: oauth2.store.ClientStore
    :param source: A callable that returns a tuple
                   (<client_id>, <client_secret>)
    :type source: callable
    """
    def __init__(self, client_store, source):
        self.client_store = client_store
        self.source = source

    def by_identifier(self, request):
        """
        Authenticates a client by its identifier.

        :param request: The incoming request
        :type request: oauth2.web.Request

        :return: The identified client
        :rtype: oauth2.datatype.Client

        :raises: :class OAuthInvalidNoRedirectError:
        """
        client_id = request.get_param("client_id")

        if client_id is None:
            raise OAuthInvalidNoRedirectError(error="missing_client_id")

        try:
            client = self.client_store.fetch_by_client_id(client_id)
        except ClientNotFoundError:
            raise OAuthInvalidNoRedirectError(error="unknown_client")

        redirect_uri = request.get_param("redirect_uri")
        if redirect_uri is not None:
            try:
                client.redirect_uri = redirect_uri
            except RedirectUriUnknown:
                raise OAuthInvalidNoRedirectError(
                    error="invalid_redirect_uri")

        return client

    def by_identifier_secret(self, request):
        """
        Authenticates a client by its identifier and secret (aka password).

        :param request: The incoming request
        :type request: oauth2.web.Request

        :return: The identified client
        :rtype: oauth2.datatype.Client

        :raises OAuthInvalidError: If the client could not be found, is not
                                   allowed to to use the current grant or
                                   supplied invalid credentials
        """
        client_id, client_secret = self.source(request=request)

        try:
            client = self.client_store.fetch_by_client_id(client_id)
        except ClientNotFoundError:
            raise OAuthInvalidError(error="invalid_client",
                                    explanation="No client could be found")

        grant_type = request.post_param("grant_type")
        if client.grant_type_supported(grant_type) is False:
            raise OAuthInvalidError(error="unauthorized_client",
                                    explanation="The client is not allowed "
                                                "to use this grant type")

        if client.secret != client_secret:
            raise OAuthInvalidError(error="invalid_client",
                                    explanation="Invalid client credentials")

        return client


def request_body(request):
    """
    Extracts the credentials of a client from the
    *application/x-www-form-urlencoded* body of a request.

    Expects the client_id to be the value of the ``client_id`` parameter and
    the client_secret to be the value of the ``client_secret`` parameter.

    :param request: The incoming request
    :type request: oauth2.web.Request

    :return: A tuple in the format of `(<CLIENT ID>, <CLIENT SECRET>)`
    :rtype: tuple
    """
    client_id = request.post_param("client_id")
    if client_id is None:
        raise OAuthInvalidError(error="invalid_request",
                                explanation="Missing client identifier")

    client_secret = request.post_param("client_secret")
    if client_secret is None:
        raise OAuthInvalidError(error="invalid_request",
                                explanation="Missing client credentials")

    return client_id, client_secret


def http_basic_auth(request):
    """
    Extracts the credentials of a client using HTTP Basic Auth.

    Expects the ``client_id`` to be the username and the ``client_secret`` to
    be the password part of the Authorization header.

    :param request: The incoming request
    :type request: oauth2.web.Request

    :return: A tuple in the format of (<CLIENT ID>, <CLIENT SECRET>)`
    :rtype: tuple
    """
    auth_header = request.header("authorization")

    if auth_header is None:
        raise OAuthInvalidError(error="invalid_request",
                                explanation="Authorization header is missing")

    auth_parts = auth_header.strip().encode("latin1").split(None)

    if auth_parts[0].strip().lower() != b'basic':
        raise OAuthInvalidError(
            error="invalid_request",
            explanation="Provider supports basic authentication only")

    client_id, client_secret = b64decode(auth_parts[1]).split(b':', 1)

    return client_id.decode("latin1"), client_secret.decode("latin1")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from mock import Mock

from oauth2.datatype import AccessToken
from oauth2.store.redisdb import TokenStore
from oauth2.test import unittest


class TokenStoreTestCase(unittest.TestCase):
    def test_delete_refresh_token(self):
        refresh_token_id = "def"
        access_token = AccessToken(client_id="abc", grant_type="token",
                                   token="xyz")

        redisdb_mock = Mock(spec=["delete", "get"])
        redisdb_mock.get.return_value = bytes(json.dumps(access_token.__dict__).encode('utf-8'))

        store = TokenStore(rs=redisdb_mock)
        store.delete_refresh_token(refresh_token_id)

        self.assertEqual(1, redisdb_mock.delete.call_count)



"""
Ensures compatibility between python 2.x and python 3.x
"""

import sys
 
if sys.version_info >= (3, 0):
    from urllib.parse import parse_qs # pragma: no cover
    from urllib.parse import urlencode # pragma: no cover
    from urllib.parse import quote # pragma: no cover
else:
    from urlparse import parse_qs # pragma: no cover
    from urllib import urlencode # pragma: no cover
    from urllib import quote # pragma: no cover



from oauth2.datatype import AuthorizationCode, AccessToken
from oauth2.error import ClientNotFoundError, AuthCodeNotFound
from oauth2.store.memory import ClientStore, TokenStore
from oauth2.test import unittest

class MemoryClientStoreTestCase(unittest.TestCase):
    def test_add_client_and_fetch_by_client_id(self):
        expected_client_data = {"client_id": "abc", "client_secret": "xyz",
                                "redirect_uris": ["http://localhost"]}
        
        store = ClientStore()
        
        success = store.add_client(expected_client_data["client_id"],
                                   expected_client_data["client_secret"],
                                   expected_client_data["redirect_uris"])
        self.assertTrue(success)
        
        client = store.fetch_by_client_id("abc")
        
        self.assertEqual(client.identifier, expected_client_data["client_id"])
        self.assertEqual(client.secret, expected_client_data["client_secret"])
        self.assertEqual(client.redirect_uris, expected_client_data["redirect_uris"])
    
    def test_fetch_by_client_id_no_client(self):
        store = ClientStore()
        
        with self.assertRaises(ClientNotFoundError):
            store.fetch_by_client_id("abc")

class MemoryTokenStoreTestCase(unittest.TestCase):
    def setUp(self):
        self.access_token_data = {"client_id": "myclient",
                                  "token": "xyz",
                                  "scopes": ["foo_read", "foo_write"],
                                  "data": {"name": "test"},
                                  "grant_type": "authorization_code"}
        self.auth_code = AuthorizationCode("myclient", "abc", 100,
                                           "http://localhost",
                                           ["foo_read", "foo_write"],
                                           {"name": "test"})
        
        self.test_store = TokenStore()
    
    def test_fetch_by_code(self):
        with self.assertRaises(AuthCodeNotFound):
            self.test_store.fetch_by_code("unknown")
    
    def test_save_code_and_fetch_by_code(self):
        success = self.test_store.save_code(self.auth_code)
        self.assertTrue(success)
        
        result = self.test_store.fetch_by_code(self.auth_code.code)
        
        self.assertEqual(result, self.auth_code)
    
    def test_save_token_and_fetch_by_token(self):
        access_token = AccessToken(**self.access_token_data)
        
        success = self.test_store.save_token(access_token)
        self.assertTrue(success)
        
        result = self.test_store.fetch_by_token(access_token.token)
        
        self.assertEqual(result, access_token)



from mock import Mock, call
from oauth2.datatype import AuthorizationCode, AccessToken
from oauth2.error import AuthCodeNotFound, AccessTokenNotFound
from oauth2.store.memcache import TokenStore
from oauth2.test import unittest

class MemcacheTokenStoreTestCase(unittest.TestCase):
    def setUp(self):
        self.cache_prefix = "test"

    def _generate_test_cache_key(self, key):
        return self.cache_prefix + "_" + key

    def test_fetch_by_code(self):
        code = "abc"
        saved_data = {"client_id": "myclient", "code": code,
                      "expires_at": 100, "redirect_uri": "http://localhost",
                      "scopes": ["foo_read", "foo_write"],
                      "data": {"name": "test"}}

        mc_mock = Mock(spec=["get"])
        mc_mock.get.return_value = saved_data

        store = TokenStore(mc=mc_mock, prefix=self.cache_prefix)

        auth_code = store.fetch_by_code(code)

        mc_mock.get.assert_called_with(self._generate_test_cache_key(code))
        self.assertEqual(auth_code.client_id, saved_data["client_id"])
        self.assertEqual(auth_code.code, saved_data["code"])
        self.assertEqual(auth_code.expires_at, saved_data["expires_at"])
        self.assertEqual(auth_code.redirect_uri, saved_data["redirect_uri"])
        self.assertEqual(auth_code.scopes, saved_data["scopes"])
        self.assertEqual(auth_code.data, saved_data["data"])

    def test_fetch_by_code_no_data(self):
        mc_mock = Mock(spec=["get"])
        mc_mock.get.return_value = None

        store = TokenStore(mc=mc_mock, prefix=self.cache_prefix)

        with self.assertRaises(AuthCodeNotFound):
            store.fetch_by_code("abc")

    def test_save_code(self):
        data = {"client_id": "myclient", "code": "abc", "expires_at": 100,
                 "redirect_uri": "http://localhost",
                 "scopes": ["foo_read", "foo_write"],
                 "data": {"name": "test"}, "user_id": 1}

        auth_code = AuthorizationCode(**data)

        cache_key = self._generate_test_cache_key(data["code"])

        mc_mock = Mock(spec=["set"])

        store = TokenStore(mc=mc_mock, prefix=self.cache_prefix)

        store.save_code(auth_code)

        mc_mock.set.assert_called_with(cache_key, data)

    def test_save_token(self):
        data = {"client_id": "myclient", "token": "xyz",
                "data": {"name": "test"}, "scopes": ["foo_read", "foo_write"],
                "expires_at": None, "refresh_token": "mno", 
                "refresh_expires_at": None,
                "grant_type": "authorization_code",
                "user_id": 123}

        access_token = AccessToken(**data)

        cache_key = self._generate_test_cache_key(access_token.token)
        refresh_token_key = self._generate_test_cache_key(access_token.refresh_token)
        unique_token_key = self._generate_test_cache_key(
            "{0}_{1}_{2}".format(access_token.client_id,
                                 access_token.grant_type,
                                 access_token.user_id)
        )

        mc_mock = Mock(spec=["set"])

        store = TokenStore(mc=mc_mock, prefix=self.cache_prefix)

        store.save_token(access_token)

        mc_mock.set.assert_has_calls([call(cache_key, data),
                                      call(unique_token_key, data),
                                      call(refresh_token_key, data)])

    def test_fetch_existing_token_of_user(self):
        data = {"client_id": "myclient", "token": "xyz",
                "data": {"name": "test"}, "scopes": ["foo_read", "foo_write"],
                "expires_at": None, "refresh_token": "mno",
                "grant_type": "authorization_code",
                "user_id": 123}

        mc_mock = Mock(spec=["get"])
        mc_mock.get.return_value = data

        store = TokenStore(mc=mc_mock, prefix=self.cache_prefix)

        access_token = store.fetch_existing_token_of_user(
            client_id="myclient",
            grant_type="authorization_code",
            user_id=123)

        self.assertTrue(isinstance(access_token, AccessToken))

    def test_fetch_existing_token_of_user_no_data(self):
        mc_mock = Mock(spec=["get"])
        mc_mock.get.return_value = None

        store = TokenStore(mc=mc_mock, prefix=self.cache_prefix)

        with self.assertRaises(AccessTokenNotFound):
            store.fetch_existing_token_of_user(client_id="myclient",
                                               grant_type="authorization_code",
                                               user_id=123)



# -*- coding: utf-8 -*-
"""
`DBApi 2.0 <http://legacy.python.org/dev/peps/pep-0249/>`_ (PEP249) compatible
implementation of data stores.
"""

from oauth2.datatype import AccessToken, AuthorizationCode, Client
from oauth2.error import AccessTokenNotFound, ClientNotFoundError, \
    AuthCodeNotFound
from oauth2.store import AccessTokenStore, AuthCodeStore, ClientStore


class DatabaseStore(object):
    """
    Base class providing functionality used by a variety of store classes.
    """
    def __init__(self, connection):
        """
        Initialize a new store class.

        :param connection: An instance of a `connection class <http://legacy.python.org/dev/peps/pep-0249/#connection-objects>`_.
        """
        self.connection = connection

    def execute(self, query, *params):
        """
        Executes a query and returns the identifier of the modified row.

        :param query: The query to be executed as a `str`.
        :param params: A `tuple` of parameters that will be replaced for
                       placeholders in the query.
        :return: A `long` identifying the last altered row.
        """
        cursor = self.connection.cursor()

        try:
            cursor.execute(query, params)

            self.connection.commit()

            return cursor.lastrowid
        finally:
            cursor.close()

    def fetchone(self, query, *args):
        """
        Returns the first result of the given query.

        :param query: The query to be executed as a `str`.
        :param params: A `tuple` of parameters that will be replaced for
                       placeholders in the query.
        :return: The retrieved row with each field being one element in a
                 `tuple`.
        """
        cursor = self.connection.cursor()

        try:
            cursor.execute(query, args)

            return cursor.fetchone()
        finally:
            cursor.close()

    def fetchall(self, query, *args):
        """
        Returns all results of the given query.

        :param query: The query to be executed as a `str`.
        :param params: A `tuple` of parameters that will be replaced for
                       placeholders in the query.
        :return: A `list` of `tuple`s with each field being one element in the
                 `tuple`.
        """
        cursor = self.connection.cursor()

        try:
            cursor.execute(query, args)

            return cursor.fetchall()
        finally:
            cursor.close()


class DbApiAccessTokenStore(DatabaseStore, AccessTokenStore):
    """
    Base class of a DBApi 2.0 compatible
    :class:`oauth2.store.AccessTokenStore`.

    A concrete implementation extends this class and defines all or a subset
    of the \*_query class attributes.
    """
    #: Insert an access token.
    create_access_token_query = None
    #: Insert one entry of additional data associated with an access token.
    create_data_query = None
    #: Insert one scope associated with an access token.
    create_scope_query = None
    #: Delete an access token by its refresh token.
    delete_refresh_token_query = None
    #: Retrieve an access token by its refresh token.
    fetch_by_refresh_token_query = None
    #: Retrieve all scopes associated with an access token.
    fetch_scopes_by_access_token_query = None
    #: Retrieve all data associated with an access token.
    fetch_data_by_access_token_query = None
    #: Retrieve an access token issued to a client and user for a specific
    #: grant.
    fetch_existing_token_of_user_query = None

    def delete_refresh_token(self, refresh_token):
        """
        Deletes an access token by its refresh token.

        :param refresh_token: The refresh token of an access token as a `str`.
        """
        self.execute(self.delete_refresh_token_query, refresh_token)

    def fetch_by_refresh_token(self, refresh_token):
        """
        Retrieves an access token by its refresh token.

        :param refresh_token: The refresh token of an access token as a `str`.

        :return: An instance of :class:`oauth2.datatype.AccessToken`.

        :raises: :class:`oauth2.error.AccessTokenNotFound` if not access token
                 could be retrieved.
        """
        row = self.fetchone(self.fetch_by_refresh_token_query, refresh_token)

        if row is None:
            raise AccessTokenNotFound

        scopes = self._fetch_scopes(access_token_id=row[0])

        data = self._fetch_data(access_token_id=row[0])

        return self._row_to_token(data=data, scopes=scopes, row=row)

    def fetch_existing_token_of_user(self, client_id, grant_type, user_id):
        """
        Retrieve an access token issued to a client and user for a specific
        grant.

        :param client_id: The identifier of a client as a `str`.
        :param grant_type: The type of grant.
        :param user_id: The identifier of the user the access token has been
                        issued to.

        :return: An instance of :class:`oauth2.datatype.AccessToken`.

        :raises: :class:`oauth2.error.AccessTokenNotFound` if not access token
                 could be retrieved.
        """
        token_data = self.fetchone(self.fetch_existing_token_of_user_query,
                                   client_id, grant_type, user_id)

        if token_data is None:
            raise AccessTokenNotFound

        scopes = self._fetch_scopes(access_token_id=token_data[0])

        data = self._fetch_data(access_token_id=token_data[0])

        return self._row_to_token(data=data, scopes=scopes, row=token_data)

    def save_token(self, access_token):
        """
        Creates a new entry for an access token in the database.

        :param access_token: An instance of
                             :class:`oauth2.datatype.AccessToken`.

         :return: `True`.
        """
        access_token_id = self.execute(self.create_access_token_query,
                                       access_token.client_id,
                                       access_token.grant_type,
                                       access_token.token,
                                       access_token.expires_at,
                                       access_token.refresh_token,
                                       access_token.refresh_expires_at,
                                       access_token.user_id)

        for key, value in list(access_token.data.items()):
            self.execute(self.create_data_query, key, value,
                         access_token_id)

        for scope in access_token.scopes:
            self.execute(self.create_scope_query, scope, access_token_id)

        return True

    def _fetch_data(self, access_token_id):
        result = self.fetchall(self.fetch_data_by_access_token_query,
                               access_token_id)
        data = {}
        if result is not None:
            for dataset in result:
                data[dataset[0]] = dataset[1]
        return data

    def _fetch_scopes(self, access_token_id):
        result = self.fetchall(self.fetch_scopes_by_access_token_query,
                               access_token_id)
        scopes = []
        if result is not None:
            for scope_data in result:
                scopes.append(scope_data[0])

        return scopes

    def _row_to_token(self, data, scopes, row):
        return AccessToken(client_id=row[1], grant_type=row[2], token=row[3],
                           data=data, expires_at=row[4], refresh_token=row[5],
                           refresh_expires_at=row[6], scopes=scopes,
                           user_id=row[7])


class DbApiAuthCodeStore(DatabaseStore, AuthCodeStore):
    """
    Base class of a DBApi 2.0 compatible :class:`oauth2.store.AuthCodeStore`.

    A concrete implementation extends this class and defines all or a subset
    of the \*_query class attributes.
    """
    #: Insert an auth code.
    create_auth_code_query = None
    #: Insert one entry of additional data associated with an auth code.
    create_data_query = None
    #: Insert one scope associated with an auth code.
    create_scope_query = None
    #: Delete an auth code.
    delete_code_query = None
    #: Retrieve an auth code by its code.
    fetch_code_query = None
    #: Retrieve all data associated with an auth code.
    fetch_data_query = None
    #: Retrieve all scopes associated with an auth code.
    fetch_scopes_query = None

    def delete_code(self, code):
        """
        Delete an auth code identified by its code.

        :param code: The code of an auth code.
        """
        self.execute(self.delete_code_query, code)

    def fetch_by_code(self, code):
        """
        Retrieves an auth code by its code.

        :param code: The code of an auth code.

        :return: An instance of :class:`oauth2.datatype.AuthorizationCode`.

        :raises: :class:`oauth2.error.AuthCodeNotFound` if no auth code could
                 be retrieved.
        """
        auth_code_data = self.fetchone(self.fetch_code_query, code)

        if auth_code_data is None:
            raise AuthCodeNotFound

        data = dict()
        data_result = self.fetchall(self.fetch_data_query, auth_code_data[0])
        if data_result is not None:
            for dataset in data_result:
                data[dataset[0]] = dataset[1]

        scopes = []
        scope_result = self.fetchall(self.fetch_scopes_query,
                                     auth_code_data[0])
        if scope_result is not None:
            for scope_set in scope_result:
                scopes.append(scope_set[0])

        return AuthorizationCode(client_id=auth_code_data[1],
                                 code=auth_code_data[2],
                                 expires_at=auth_code_data[3],
                                 redirect_uri=auth_code_data[4],
                                 scopes=scopes, data=data,
                                 user_id=auth_code_data[5])

    def save_code(self, authorization_code):
        """
        Creates a new entry of an auth code in the database.

        :param authorization_code: An instance of
                                   :class:`oauth2.datatype.AuthorizationCode`.

        :return: `True` if everything went fine.
        """
        auth_code_id = self.execute(self.create_auth_code_query,
                                    authorization_code.client_id,
                                    authorization_code.code,
                                    authorization_code.expires_at,
                                    authorization_code.redirect_uri,
                                    authorization_code.user_id)

        for key, value in list(authorization_code.data.items()):
            self.execute(self.create_data_query, key, value, auth_code_id)

        for scope in authorization_code.scopes:
            self.execute(self.create_scope_query, scope, auth_code_id)

        return True


class DbApiClientStore(DatabaseStore, ClientStore):
    """
    Base class of a DBApi 2.0 compatible :class:`oauth2.store.ClientStore`.

    A concrete implementation extends this class and defines all or a subset
    of the \*_query class attributes.
    """
    #: Retrieve a client by its identifier.
    fetch_client_query = None
    #: Retrieve all grants that a client is allowed to use.
    fetch_grants_query = None
    #: Retrieve all redirect URIs of a client.
    fetch_redirect_uris_query = None
    #: Retrieve all response types that a client supports.
    fetch_response_types_query = None

    def fetch_by_client_id(self, client_id):
        """
        Retrieves a client by its identifier.

        :param client_id: The identifier of a client.

        :return: An instance of :class:`oauth2.datatype.Client`.

        :raises: :class:`oauth2.error.ClientError` if no client could be
                 retrieved.
        """
        grants = None
        redirect_uris = None
        response_types = None

        client_data = self.fetchone(self.fetch_client_query, client_id)

        if client_data is None:
            raise ClientNotFoundError

        grant_data = self.fetchall(self.fetch_grants_query, client_data[0])
        if grant_data:
            grants = []
            for grant in grant_data:
                grants.append(grant[0])

        redirect_uris_data = self.fetchall(self.fetch_redirect_uris_query,
                                           client_data[0])
        if redirect_uris_data:
            redirect_uris = []
            for redirect_uri in redirect_uris_data:
                redirect_uris.append(redirect_uri[0])

        response_types_data = self.fetchall(self.fetch_response_types_query,
                                            client_data[0])
        if response_types_data:
            response_types = []
            for response_type in response_types_data:
                response_types.append(response_type[0])

        return Client(identifier=client_data[1], secret=client_data[2],
                      authorized_grants=grants,
                      authorized_response_types=response_types,
                      redirect_uris=redirect_uris)



"""
=============
python-oauth2
=============

python-oauth2 is a framework that aims at making it easy to provide
authentication via `OAuth 2.0 <http://tools.ietf.org/html/rfc6749>`_ within
an application stack.

Usage
=====

Example:

.. literalinclude:: examples/base_server.py

Installation
============

python-oauth2 is available on
`PyPI <http://pypi.python.org/pypi/python-oauth2/>`_::

    pip install python-oauth2

"""

import json
from oauth2.client_authenticator import ClientAuthenticator, request_body
from oauth2.error import OAuthInvalidError, \
    ClientNotFoundError, OAuthInvalidNoRedirectError, UnsupportedGrantError
from oauth2.log import app_log
from oauth2.web import Response
from oauth2.tokengenerator import Uuid4
from oauth2.grant import Scope, AuthorizationCodeGrant, ImplicitGrant, \
    ClientCredentialsGrant, ResourceOwnerGrant, RefreshToken

VERSION = "1.1.0"


class Provider(object):
    """
    Endpoint of requests to the OAuth 2.0 provider.

    :param access_token_store: An object that implements methods defined
                               by :class:`oauth2.store.AccessTokenStore`.
    :type access_token_store: oauth2.store.AccessTokenStore
    :param auth_code_store: An object that implements methods defined by
                            :class:`oauth2.store.AuthCodeStore`.
    :type auth_code_store: oauth2.store.AuthCodeStore
    :param client_store: An object that implements methods defined by
                         :class:`oauth2.store.ClientStore`.
    :type client_store: oauth2.store.ClientStore
    :param token_generator: Object to generate unique tokens.
    :type token_generator: oauth2.tokengenerator.TokenGenerator
    :param client_authentication_source: A callable which when executed,
                                         authenticates a client.
                                         See :mod:`oauth2.client_authenticator`.
    :type client_authentication_source: callable
    :param response_class: Class of the response object.
                           Defaults to :class:`oauth2.web.Response`.
    :type response_class: oauth2.web.Response

    .. versionchanged:: 1.0.0
       Removed parameter ``site_adapter``.
    """
    authorize_path = "/authorize"
    token_path = "/token"

    def __init__(self, access_token_store, auth_code_store, client_store,
                 token_generator, client_authentication_source=request_body,
                 response_class=Response):
        self.grant_types = []
        self._input_handler = None

        self.access_token_store = access_token_store
        self.auth_code_store = auth_code_store
        self.client_authenticator = ClientAuthenticator(
            client_store=client_store,
            source=client_authentication_source)
        self.response_class = response_class
        self.token_generator = token_generator

    def add_grant(self, grant):
        """
        Adds a Grant that the provider should support.

        :param grant: An instance of a class that extends
                      :class:`oauth2.grant.GrantHandlerFactory`
        :type grant: oauth2.grant.GrantHandlerFactory
        """
        if hasattr(grant, "expires_in"):
            self.token_generator.expires_in[grant.grant_type] = grant.expires_in

        if hasattr(grant, "refresh_expires_in"):
            self.token_generator.refresh_expires_in = grant.refresh_expires_in

        self.grant_types.append(grant)

    def dispatch(self, request, environ):
        """
        Checks which Grant supports the current request and dispatches to it.

        :param request: The incoming request.
        :type request: :class:`oauth2.web.Request`
        :param environ: Dict containing variables of the environment.
        :type environ: dict

        :return: An instance of ``oauth2.web.Response``.
        """
        try:
            grant_type = self._determine_grant_type(request)

            response = self.response_class()

            grant_type.read_validate_params(request)

            return grant_type.process(request, response, environ)
        except OAuthInvalidNoRedirectError:
            response = self.response_class()
            response.add_header("Content-Type", "application/json")
            response.status_code = 400
            response.body = json.dumps({
                "error": "invalid_redirect_uri",
                "error_description": "Invalid redirect URI"
            })

            return response
        except OAuthInvalidError as err:
            response = self.response_class()
            return grant_type.handle_error(error=err, response=response)
        except UnsupportedGrantError:
            response = self.response_class()
            response.add_header("Content-Type", "application/json")
            response.status_code = 400
            response.body = json.dumps({
                "error": "unsupported_response_type",
                "error_description": "Grant not supported"
            })

            return response
        except:
            app_log.error("Uncaught Exception", exc_info=True)
            response = self.response_class()
            return grant_type.handle_error(
                error=OAuthInvalidError(error="server_error",
                                        explanation="Internal server error"),
                response=response)

    def enable_unique_tokens(self):
        """
        Enable the use of unique access tokens on all grant types that support
        this option.
        """
        for grant_type in self.grant_types:
            if hasattr(grant_type, "unique_token"):
                grant_type.unique_token = True

    @property
    def scope_separator(self, separator):
        """
        Sets the separator of values in the scope query parameter.
        Defaults to " " (whitespace).

        The following code makes the Provider use "," instead of " "::

            provider = Provider()

            provider.scope_separator = ","

        Now the scope parameter in the request of a client can look like this:
        `scope=foo,bar`.
        """
        Scope.separator = separator

    def _determine_grant_type(self, request):
        for grant in self.grant_types:
            grant_handler = grant(request, self)
            if grant_handler is not None:
                return grant_handler

        raise UnsupportedGrantError



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. warning::

   Tornado support is currently experimental.

Use Tornado to serve token requests:

.. literalinclude:: examples/tornado_server.py
"""

from __future__ import absolute_import

from tornado.web import RequestHandler


class Request(object):
    def __init__(self, handler):
        """
        :param handler: Handler of the current request
        :type handler: :class:`tornado.web.RequestHandler`
        """
        self.handler = handler

    @property
    def method(self):
        return self.handler.request.method

    @property
    def path(self):
        return self.handler.request.path

    @property
    def query_string(self):
        return self.handler.request.query

    def get_param(self, name, default=None):
        return self.handler.get_query_argument(name=name, default=default)

    def header(self, name, default=None):
        return self.handler.request.headers[name]

    def post_param(self, name, default=None):
        return self.handler.get_body_argument(name=name, default=default)


class OAuth2Handler(RequestHandler):
    def initialize(self, provider):
        """
        :type provider: :class:`oauth2.Provider`
        """
        self.provider = provider

    def get(self):
        response = self._dispatch_request()

        self._map_response(response)

    def post(self):
        response = self._dispatch_request()

        self._map_response(response)

    def _dispatch_request(self):
        return self.provider.dispatch(request=Request(handler=self),
                                      environ=dict())

    def _map_response(self, response):
        for name, value in list(response.headers.items()):
            self.set_header(name, value)

        self.set_status(response.status_code)
        self.write(response.body)



import json
from mock import Mock
from oauth2.error import OAuthInvalidNoRedirectError, OAuthInvalidError
from oauth2.test import unittest
from oauth2 import Provider
from oauth2.store import ClientStore
from oauth2.web import Response, AuthorizationCodeGrantSiteAdapter, \
    ResourceOwnerGrantSiteAdapter
from oauth2.web.wsgi import Request
from oauth2.grant import RefreshToken, AuthorizationCodeGrant, GrantHandler, \
    ResourceOwnerGrant


class ProviderTestCase(unittest.TestCase):
    def setUp(self):
        self.client_store_mock = Mock(spec=ClientStore)
        self.token_generator_mock = Mock()

        self.response_mock = Mock(spec=Response)
        self.response_mock.body = ""
        response_class_mock = Mock(return_value=self.response_mock)

        self.token_generator_mock.expires_in = {}
        self.token_generator_mock.refresh_expires_in = 0

        self.auth_server = Provider(access_token_store=Mock(),
                                    auth_code_store=Mock(),
                                    client_store=self.client_store_mock,
                                    token_generator=self.token_generator_mock,
                                    response_class=response_class_mock)

    def test_add_grant_set_expire_time(self):
        """
        Provider.add_grant() should set the expiration time on the instance of TokenGenerator
        """
        self.auth_server.add_grant(
            AuthorizationCodeGrant(
                expires_in=400,
                site_adapter=Mock(spec=AuthorizationCodeGrantSiteAdapter)
            )
        )
        self.auth_server.add_grant(
            ResourceOwnerGrant(
                expires_in=500,
                site_adapter=Mock(spec=ResourceOwnerGrantSiteAdapter)
            )
        )
        self.auth_server.add_grant(RefreshToken(expires_in=1200))

        self.assertEqual(self.token_generator_mock.expires_in[AuthorizationCodeGrant.grant_type], 400)
        self.assertEqual(self.token_generator_mock.expires_in[ResourceOwnerGrant.grant_type], 500)
        self.assertEqual(self.token_generator_mock.refresh_expires_in, 1200)

    def test_dispatch(self):
        environ = {"session": "data"}
        process_result = "response"

        request_mock = Mock(spec=Request)

        grant_handler_mock = Mock(spec=["process", "read_validate_params"])
        grant_handler_mock.process.return_value = process_result

        grant_factory_mock = Mock(return_value=grant_handler_mock)

        self.auth_server.site_adapter = Mock(
            spec=AuthorizationCodeGrantSiteAdapter
        )
        self.auth_server.add_grant(grant_factory_mock)
        result = self.auth_server.dispatch(request_mock, environ)

        grant_factory_mock.assert_called_with(request_mock, self.auth_server)
        grant_handler_mock.read_validate_params.\
            assert_called_with(request_mock)
        grant_handler_mock.process.assert_called_with(request_mock,
                                                      self.response_mock,
                                                      environ)
        self.assertEqual(result, process_result)

    def test_dispatch_no_grant_type_found(self):
        error_body = {
            "error": "unsupported_response_type",
            "error_description": "Grant not supported"
        }

        request_mock = Mock(spec=Request)

        result = self.auth_server.dispatch(request_mock, {})

        self.response_mock.add_header.assert_called_with("Content-Type",
                                                         "application/json")
        self.assertEqual(self.response_mock.status_code, 400)
        self.assertEqual(self.response_mock.body, json.dumps(error_body))
        self.assertEqual(result, self.response_mock)

    def test_dispatch_no_client_found(self):
        request_mock = Mock(spec=Request)

        grant_handler_mock = Mock(spec=GrantHandler)
        grant_handler_mock.process.side_effect = OAuthInvalidNoRedirectError(
            error="")

        grant_factory_mock = Mock(return_value=grant_handler_mock)

        self.auth_server.add_grant(grant_factory_mock)
        self.auth_server.dispatch(request_mock, {})

        self.response_mock.add_header.assert_called_with("Content-Type",
                                                         "application/json")
        self.assertEqual(self.response_mock.status_code, 400)
        self.assertEqual(json.loads(self.response_mock.body)["error"], "invalid_redirect_uri")

    def test_dispatch_general_exception(self):
        request_mock = Mock(spec=Request)

        grant_handler_mock = Mock(spec=GrantHandler)
        grant_handler_mock.process.side_effect = KeyError

        grant_factory_mock = Mock(return_value=grant_handler_mock)

        self.auth_server.add_grant(grant_factory_mock)
        self.auth_server.dispatch(request_mock, {})

        self.assertTrue(grant_handler_mock.handle_error.called)



from oauth2.test import unittest
from oauth2.store.mongodb import AccessTokenStore, AuthCodeStore, \
    ClientStore
from mock import Mock
from oauth2.datatype import AccessToken, AuthorizationCode, Client
from oauth2.error import AccessTokenNotFound, AuthCodeNotFound, \
    ClientNotFoundError


class MongodbAccessTokenStoreTestCase(unittest.TestCase):
    def setUp(self):
        self.access_token_data = {"client_id": "myclient",
                                  "grant_type": "authorization_code",
                                  "token": "xyz",
                                  "scopes": ["foo_read", "foo_write"],
                                  "data": {"name": "test"},
                                  "expires_at": 1000,
                                  "refresh_token": "abcd",
                                  "refresh_expires_at": 2000,
                                  "user_id": None}

    def test_fetch_by_refresh_token(self):
        refresh_token = "abcd"

        self.access_token_data["refresh_token"] = refresh_token

        collection_mock = Mock(spec=["find_one"])
        collection_mock.find_one.return_value = self.access_token_data

        store = AccessTokenStore(collection=collection_mock)
        token = store.fetch_by_refresh_token(refresh_token=refresh_token)

        collection_mock.find_one.assert_called_with(
            {"refresh_token": refresh_token})
        self.assertTrue(isinstance(token, AccessToken))
        self.assertDictEqual(token.__dict__, self.access_token_data)

    def test_fetch_by_refresh_token_no_data(self):
        collection_mock = Mock(spec=["find_one"])
        collection_mock.find_one.return_value = None

        store = AccessTokenStore(collection=collection_mock)

        with self.assertRaises(AccessTokenNotFound):
            store.fetch_by_refresh_token(refresh_token="abcd")

    def test_fetch_existing_token_of_user(self):
        test_data = {"client_id": "myclient",
                     "grant_type": "authorization_code",
                     "token": "xyz",
                     "scopes": ["foo_read", "foo_write"],
                     "data": {"name": "test"},
                     "expires_at": 1000,
                     "refresh_token": "abcd",
                     "refresh_expires_at": 2000,
                     "user_id": 123}

        collection_mock = Mock(spec=["find_one"])
        collection_mock.find_one.return_value = test_data

        store = AccessTokenStore(collection=collection_mock)

        token = store.fetch_existing_token_of_user(client_id="myclient",
                                                   grant_type="authorization_code",
                                                   user_id=123)

        self.assertTrue(isinstance(token, AccessToken))
        self.assertDictEqual(token.__dict__, test_data)
        collection_mock.find_one.assert_called_with({"client_id": "myclient",
                                                     "grant_type": "authorization_code",
                                                     "user_id": 123},
                                                    sort=[("expires_at", -1)])

    def test_fetch_existing_token_of_user_no_data(self):
        collection_mock = Mock(spec=["find_one"])
        collection_mock.find_one.return_value = None

        store = AccessTokenStore(collection=collection_mock)

        with self.assertRaises(AccessTokenNotFound):
            store.fetch_existing_token_of_user(client_id="myclient",
                                               grant_type="authorization_code",
                                               user_id=123)

    def test_save_token(self):
        access_token = AccessToken(**self.access_token_data)

        collection_mock = Mock(spec=["insert"])

        store = AccessTokenStore(collection=collection_mock)
        store.save_token(access_token)

        collection_mock.insert.assert_called_with(self.access_token_data)

class MongodbAuthCodeStoreTestCase(unittest.TestCase):
    def setUp(self):
        self.auth_code_data = {"client_id": "myclient", "expires_at": 1000,
                               "redirect_uri": "https://redirect",
                               "scopes": ["foo", "bar"], "data": {},
                               "user_id": None}

        self.collection_mock = Mock(spec=["find_one", "insert", "remove"])

    def test_fetch_by_code(self):
        code = "abcd"

        self.collection_mock.find_one.return_value = self.auth_code_data

        self.auth_code_data["code"] = "abcd"

        store = AuthCodeStore(collection=self.collection_mock)
        auth_code = store.fetch_by_code(code=code)

        self.collection_mock.find_one.assert_called_with({"code": "abcd"})
        self.assertTrue(isinstance(auth_code, AuthorizationCode))
        self.assertDictEqual(auth_code.__dict__, self.auth_code_data)

    def test_fetch_by_code_no_data(self):
        self.collection_mock.find_one.return_value = None

        store = AuthCodeStore(collection=self.collection_mock)

        with self.assertRaises(AuthCodeNotFound):
            store.fetch_by_code(code="abcd")

    def test_save_code(self):
        self.auth_code_data["code"] = "abcd"

        auth_code = AuthorizationCode(**self.auth_code_data)

        store = AuthCodeStore(collection=self.collection_mock)
        store.save_code(auth_code)

        self.collection_mock.insert.assert_called_with(self.auth_code_data)

class MongodbClientStoreTestCase(unittest.TestCase):
    def test_fetch_by_client_id(self):
        client_data = {"identifier": "testclient", "secret": "k#4g6",
                       "redirect_uris": ["https://redirect"],
                       "authorized_grants": []}

        collection_mock = Mock(spec=["find_one"])
        collection_mock.find_one.return_value = client_data

        store = ClientStore(collection=collection_mock)
        client = store.fetch_by_client_id(client_id=client_data["identifier"])

        collection_mock.find_one.assert_called_with({
            "identifier": client_data["identifier"]})
        self.assertTrue(isinstance(client, Client))
        self.assertEqual(client.identifier, client_data["identifier"])

    def test_fetch_by_client_id_no_data(self):
        collection_mock = Mock(spec=["find_one"])
        collection_mock.find_one.return_value = None

        store = ClientStore(collection=collection_mock)

        with self.assertRaises(ClientNotFoundError):
            store.fetch_by_client_id(client_id="testclient")



import os
import redis
from wsgiref.simple_server import WSGIRequestHandler
from pymongo import MongoClient
import mysql.connector
import oauth2.store.mongodb
import oauth2.store.memory
import oauth2.store.dbapi.mysql
import oauth2.store.redisdb


class NoLoggingHandler(WSGIRequestHandler):
    """
    Turn off logging access to STDERR in the standard WSGI request handler.
    """
    def log_message(self, format, *args):
        pass


def store_factory(client_identifier, client_secret, redirect_uris):
    stores = {"access_token_store": None, "auth_code_store": None,
              "client_store": None}

    database = os.environ.get("DB")

    if database == "mongodb":
        creator_class = MongoDbStoreCreator
    elif database == "mysql":
        creator_class = MysqlStoreCreator
    elif database == "redis-server":
        creator_class = RedisStoreCreator
    else:
        creator_class = MemoryStoreCreator

    creator = creator_class(client_identifier, client_secret, redirect_uris)

    creator.initialize()

    creator.before_create()

    stores["access_token_store"] = creator.create_access_token_store()
    stores["auth_code_store"] = creator.create_auth_code_store()
    stores["client_store"] = creator.create_client_store()

    creator.after_create()

    return stores


class StoreCreator(object):
    def __init__(self, client_identifier, client_secret, redirect_uris):
        self.client_identifier = client_identifier
        self.client_secret = client_secret
        self.redirect_uris = redirect_uris

    def initialize(self):
        pass

    def after_create(self):
        pass

    def before_create(self):
        pass

    def create_access_token_store(self):
        raise NotImplementedError

    def create_auth_code_store(self):
        raise NotImplementedError

    def create_client_store(self):
        raise NotImplementedError


class MemoryStoreCreator(StoreCreator):
    def initialize(self):
        self.client_store = oauth2.store.memory.ClientStore()
        self.token_store = oauth2.store.memory.TokenStore()

    def create_access_token_store(self):
        return self.token_store

    def create_auth_code_store(self):
        return self.token_store

    def create_client_store(self):
        return self.client_store

    def after_create(self):
        self.client_store.add_client(client_id=self.client_identifier,
                                     client_secret=self.client_secret,
                                     redirect_uris=self.redirect_uris)


class MongoDbStoreCreator(StoreCreator):
    def initialize(self):
        client = MongoClient('127.0.0.1', 27017)

        self.db = client.test_database

    def create_access_token_store(self):
        return oauth2.store.mongodb.AccessTokenStore(
            collection=self.db["access_tokens"]
        )

    def create_auth_code_store(self):
        return oauth2.store.mongodb.AuthCodeStore(
            collection=self.db["auth_codes"]
        )

    def create_client_store(self):
        return oauth2.store.mongodb.ClientStore(collection=self.db["clients"])

    def after_create(self):
        self.db["clients"].insert({
            "identifier": "abc",
            "secret": "xyz",
            "redirect_uris": ["http://127.0.0.1:15487/callback"]
        })


class MysqlStoreCreator(StoreCreator):
    create_tables = """
DROP TABLE IF EXISTS `access_tokens`;
CREATE TABLE IF NOT EXISTS `access_tokens` (
  `id` INT NOT NULL AUTO_INCREMENT COMMENT 'Unique identifier',
  `client_id` VARCHAR(32) NOT NULL COMMENT 'The identifier of a client. Assuming it is an arbitrary text which is a maximum of 32 characters long.',
  `grant_type` ENUM('authorization_code', 'implicit', 'password', 'client_credentials', 'refresh_token') NOT NULL COMMENT 'The type of a grant for which a token has been issued.',
  `token` CHAR(36) NOT NULL COMMENT 'The access token.',
  `expires_at` TIMESTAMP NULL COMMENT 'The timestamp at which the token expires.',
  `refresh_token` CHAR(36) NULL COMMENT 'The refresh token.',
  `refresh_expires_at` TIMESTAMP NULL COMMENT 'The timestamp at which the refresh token expires.',
  `user_id` INT NULL COMMENT 'The identifier of the user this token belongs to.',
  PRIMARY KEY (`id`),
  INDEX `fetch_by_refresh_token` (`refresh_token` ASC),
  INDEX `fetch_existing_token_of_user` (`client_id` ASC, `grant_type` ASC, `user_id` ASC))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `access_token_scopes`;
CREATE TABLE IF NOT EXISTS `access_token_scopes` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `name` VARCHAR(32) NOT NULL COMMENT 'The name of scope.',
  `access_token_id` INT NOT NULL COMMENT 'The unique identifier of the access token this scope belongs to.',
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `access_token_data`;
CREATE TABLE IF NOT EXISTS `access_token_data` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `key` VARCHAR(32) NOT NULL COMMENT 'The key of an entry converted to the key in a Python dict.',
  `value` VARCHAR(32) NOT NULL COMMENT 'The value of an entry converted to the value in a Python dict.',
  `access_token_id` INT NOT NULL COMMENT 'The unique identifier of the access token a row  belongs to.',
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `auth_codes`;
CREATE TABLE IF NOT EXISTS `auth_codes` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `client_id` VARCHAR(32) NOT NULL COMMENT 'The identifier of a client. Assuming it is an arbitrary text which is a maximum of 32 characters long.',
  `code` CHAR(36) NOT NULL COMMENT 'The authorisation code.',
  `expires_at` TIMESTAMP NOT NULL COMMENT 'The timestamp at which the token expires.',
  `redirect_uri` VARCHAR(128) NULL COMMENT 'The redirect URI send by the client during the request of an authorisation code.',
  `user_id` INT NULL COMMENT 'The identifier of the user this authorisation code belongs to.',
  PRIMARY KEY (`id`),
  INDEX `fetch_code` (`code` ASC))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `auth_code_data`;
CREATE TABLE IF NOT EXISTS `auth_code_data` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `key` VARCHAR(32) NOT NULL COMMENT 'The key of an entry converted to the key in a Python dict.',
  `value` VARCHAR(32) NOT NULL COMMENT 'The value of an entry converted to the value in a Python dict.',
  `auth_code_id` INT NOT NULL COMMENT 'The identifier of the authorisation code that this row belongs to.',
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `auth_code_scopes`;
CREATE TABLE IF NOT EXISTS `auth_code_scopes` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `name` VARCHAR(32) NOT NULL,
  `auth_code_id` INT NOT NULL,
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `clients`;
CREATE TABLE IF NOT EXISTS `clients` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `identifier` VARCHAR(32) NOT NULL COMMENT 'The identifier of a client.',
  `secret` VARCHAR(32) NOT NULL COMMENT 'The secret of a client.',
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `client_grants`;
CREATE TABLE IF NOT EXISTS `client_grants` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `name` VARCHAR(32) NOT NULL,
  `client_id` INT NOT NULL COMMENT 'The id of the client a row belongs to.',
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `client_redirect_uris`;
CREATE TABLE IF NOT EXISTS `client_redirect_uris` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `redirect_uri` VARCHAR(128) NOT NULL COMMENT 'A URI of a client.',
  `client_id` INT NOT NULL COMMENT 'The id of the client a row belongs to.',
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

DROP TABLE IF EXISTS `client_response_types`;
CREATE TABLE IF NOT EXISTS `client_response_types` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `response_type` VARCHAR(32) NOT NULL COMMENT 'The response type that a client can use.',
  `client_id` INT NOT NULL COMMENT 'The id of the client a row belongs to.',
  PRIMARY KEY (`id`))
ENGINE = InnoDB;"""

    def initialize(self):
        self.connection = mysql.connector.connect(host="127.0.0.1",
                                                  user="root", passwd="",
                                                  db="testdb")

    def create_access_token_store(self):
        return oauth2.store.dbapi.mysql.\
            MysqlAccessTokenStore(connection=self.connection)

    def create_auth_code_store(self):
        return oauth2.store.dbapi.mysql.\
            MysqlAuthCodeStore(connection=self.connection)

    def create_client_store(self):
        return oauth2.store.dbapi.mysql.\
            MysqlClientStore(connection=self.connection)

    def before_create(self):
        # Execute each query on its own instead of one big query.
        # The one big query caused errors where some tables were not created
        # in every run of the tests.
        for stmt in self.create_tables.split(';'):
            cursor = self.connection.cursor()

            try:
                cursor.execute(stmt)

                self.connection.commit()
            finally:
                cursor.close()

    def after_create(self):
        cursor = self.connection.cursor()

        try:
            cursor.execute(
                "INSERT INTO clients (identifier, secret) VALUES (%s, %s)",
                ("abc", "xyz"))

            client_row_id = cursor.lastrowid

            self.connection.commit()
        finally:
            cursor.close()

        cursor = self.connection.cursor()

        try:
            cursor.execute(
                """INSERT INTO client_redirect_uris
                    (redirect_uri, client_id)
                VALUES (%s, %s)""",
                ("http://127.0.0.1:15487/callback", client_row_id)
            )

            self.connection.commit()
        finally:
            cursor.close()


class RedisStoreCreator(StoreCreator):
    def initialize(self):
        self.r = redis.StrictRedis(host="localhost", port=6379, db=0)

        self.client_store = oauth2.store.redisdb.ClientStore(rs=self.r)
        self.token_store = oauth2.store.redisdb.TokenStore(rs=self.r)

    def create_access_token_store(self):
        return self.token_store

    def create_auth_code_store(self):
        return self.token_store

    def create_client_store(self):
        return self.client_store

    def after_create(self):
        self.client_store.add_client(client_id=self.client_identifier,
                                     client_secret=self.client_secret,
                                     redirect_uris=self.redirect_uris)



# -*- coding: utf-8 -*-
"""
Adapters to use mysql as the storage backend.

This module uses the API defined in :mod:`oauth2.store.dbapi`.
Therefore no logic is defined here. Instead all classes define the queries
required by :mod:`oauth2.store.dbapi`.

The queries have been created for the following SQL tables in mind:

.. code-block:: sql

    CREATE TABLE IF NOT EXISTS `testdb`.`access_tokens` (
      `id` INT NOT NULL AUTO_INCREMENT COMMENT 'Unique identifier',
      `client_id` VARCHAR(32) NOT NULL COMMENT 'The identifier of a client. Assuming it is an arbitrary text which is a maximum of 32 characters long.',
      `grant_type` ENUM('authorization_code', 'implicit', 'password', 'client_credentials', 'refresh_token') NOT NULL COMMENT 'The type of a grant for which a token has been issued.',
      `token` CHAR(36) NOT NULL COMMENT 'The access token.',
      `expires_at` TIMESTAMP NULL COMMENT 'The timestamp at which the token expires.',
      `refresh_token` CHAR(36) NULL COMMENT 'The refresh token.',
      `refresh_expires_at` TIMESTAMP NULL COMMENT 'The timestamp at which the refresh token expires.',
      `user_id` INT NULL COMMENT 'The identifier of the user this token belongs to.',
      PRIMARY KEY (`id`),
      INDEX `fetch_by_refresh_token` (`refresh_token` ASC),
      INDEX `fetch_existing_token_of_user` (`client_id` ASC, `grant_type` ASC, `user_id` ASC))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`access_token_scopes` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `name` VARCHAR(32) NOT NULL COMMENT 'The name of scope.',
      `access_token_id` INT NOT NULL COMMENT 'The unique identifier of the access token this scope belongs to.',
      PRIMARY KEY (`id`))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`access_token_data` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `key` VARCHAR(32) NOT NULL COMMENT 'The key of an entry converted to the key in a Python dict.',
      `value` VARCHAR(32) NOT NULL COMMENT 'The value of an entry converted to the value in a Python dict.',
      `access_token_id` INT NOT NULL COMMENT 'The unique identifier of the access token a row  belongs to.',
      PRIMARY KEY (`id`))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`auth_codes` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `client_id` VARCHAR(32) NOT NULL COMMENT 'The identifier of a client. Assuming it is an arbitrary text which is a maximum of 32 characters long.',
      `code` CHAR(36) NOT NULL COMMENT 'The authorisation code.',
      `expires_at` TIMESTAMP NOT NULL COMMENT 'The timestamp at which the token expires.',
      `redirect_uri` VARCHAR(128) NULL COMMENT 'The redirect URI send by the client during the request of an authorisation code.',
      `user_id` INT NULL COMMENT 'The identifier of the user this authorisation code belongs to.',
      PRIMARY KEY (`id`),
      INDEX `fetch_code` (`code` ASC))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`auth_code_data` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `key` VARCHAR(32) NOT NULL COMMENT 'The key of an entry converted to the key in a Python dict.',
      `value` VARCHAR(32) NOT NULL COMMENT 'The value of an entry converted to the value in a Python dict.',
      `auth_code_id` INT NOT NULL COMMENT 'The identifier of the authorisation code that this row belongs to.',
      PRIMARY KEY (`id`))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`auth_code_scopes` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `name` VARCHAR(32) NOT NULL,
      `auth_code_id` INT NOT NULL,
      PRIMARY KEY (`id`))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`clients` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `identifier` VARCHAR(32) NOT NULL COMMENT 'The identifier of a client.',
      `secret` VARCHAR(32) NOT NULL COMMENT 'The secret of a client.',
      PRIMARY KEY (`id`))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`client_grants` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `name` VARCHAR(32) NOT NULL,
      `client_id` INT NOT NULL COMMENT 'The id of the client a row belongs to.',
      PRIMARY KEY (`id`))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`client_redirect_uris` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `redirect_uri` VARCHAR(128) NOT NULL COMMENT 'A URI of a client.',
      `client_id` INT NOT NULL COMMENT 'The id of the client a row belongs to.',
      PRIMARY KEY (`id`))
    ENGINE = InnoDB;

    CREATE TABLE IF NOT EXISTS `testdb`.`client_response_types` (
      `id` INT NOT NULL AUTO_INCREMENT,
      `response_type` VARCHAR(32) NOT NULL COMMENT 'The response type that a client can use.',
      `client_id` INT NOT NULL COMMENT 'The id of the client a row belongs to.',
      PRIMARY KEY (`id`))
    ENGINE = InnoDB;
"""

from oauth2.store.dbapi import DbApiAccessTokenStore, DbApiAuthCodeStore, \
    DbApiClientStore


class MysqlAccessTokenStore(DbApiAccessTokenStore):
    delete_refresh_token_query = """
        DELETE FROM
            `access_tokens`
        WHERE
            `refresh_token` = %s"""

    fetch_by_refresh_token_query = """
        SELECT
           `id`, `client_id`, `grant_type`, `token`,
           UNIX_TIMESTAMP(`expires_at`), `refresh_token`,
           UNIX_TIMESTAMP(`refresh_expires_at`), `user_id`
        FROM
            `access_tokens`
        WHERE
            `refresh_token` = %s
        LIMIT 1"""

    fetch_scopes_by_access_token_query = """
        SELECT
            `name`
        FROM
            `access_token_scopes`
        WHERE
            `access_token_id` = %s"""

    fetch_data_by_access_token_query = """
        SELECT
           `key`, `value`
        FROM
            `access_token_data`
        WHERE
            `access_token_id` = %s"""

    fetch_existing_token_of_user_query = """
        SELECT
           `id`, `client_id`, `grant_type`, `token`,
           UNIX_TIMESTAMP(`expires_at`), `refresh_token`,
           UNIX_TIMESTAMP(`refresh_expires_at`), `user_id`
        FROM
            `access_tokens`
        WHERE
            `client_id` = %s
        AND
            `grant_type` = %s
        AND
            `user_id` = %s
        ORDER BY
            `expires_at` DESC
        LIMIT 1"""

    create_access_token_query = """
        INSERT INTO `access_tokens` (
           `client_id`, `grant_type`, `token`, `expires_at`, `refresh_token`,
           `refresh_expires_at`, `user_id`
        ) VALUES (
            %s, %s, %s, FROM_UNIXTIME(%s), %s, FROM_UNIXTIME(%s), %s
        )"""

    create_data_query = """
        INSERT INTO `access_token_data` (
           `key`,`value`, `access_token_id`
        ) VALUES (
            %s, %s, %s
        )"""

    create_scope_query = """
        INSERT INTO `access_token_scopes` (
           `name`, `access_token_id`
        ) VALUES (
            %s, %s
        )"""


class MysqlAuthCodeStore(DbApiAuthCodeStore):
    create_auth_code_query = """
        INSERT INTO `auth_codes` (
           `client_id`,`code`,`expires_at`,`redirect_uri`, `user_id`
        ) VALUES (
            %s, %s, FROM_UNIXTIME(%s), %s, %s
        )"""

    create_data_query = """
        INSERT INTO `auth_code_data` (
           `key`,`value`, `auth_code_id`
        ) VALUES (
            %s, %s, %s
        )"""

    create_scope_query = """
        INSERT INTO `auth_code_scopes` (
           `name`, `auth_code_id`
        ) VALUES (
            %s, %s
        )"""

    delete_code_query = """
        DELETE FROM `auth_codes` WHERE code = %s"""

    fetch_code_query = """
        SELECT
            `id`, `client_id`, `code`, UNIX_TIMESTAMP(`expires_at`),
            `redirect_uri`, `user_id`
        FROM
            `auth_codes`
        WHERE
            `code` = %s"""

    fetch_data_query = """
        SELECT
            `key`, `value`
        FROM
            `auth_code_data`
        WHERE
            `auth_code_id` = %s"""

    fetch_scopes_query = """
        SELECT
            `name`
        FROM
            `auth_code_scopes`
        WHERE
            `auth_code_id` = %s"""


class MysqlClientStore(DbApiClientStore):
    fetch_client_query = """
        SELECT
           `id`,`identifier`, `secret`
        FROM
            `clients`
        WHERE
            `identifier` = %s"""

    fetch_grants_query = """
        SELECT
            `name`
        FROM
            `client_grants`
        WHERE
            `client_id` = %s"""
    fetch_redirect_uris_query = """
        SELECT
            `redirect_uri`
        FROM
            `client_redirect_uris`
        WHERE
            `client_id` = %s"""

    fetch_response_types_query = """
        SELECT
            `response_type`
        FROM
            `client_response_types`
        WHERE
            `client_id` = %s"""



"""
Logging support

There are two loggers available:

* ``oauth2.application``: Logging of uncaught exceptions
* ``oauth2.general``: General purpose logging of debug errors and warnings

If logging has not been configured, you will likely see this error:

.. code-block:: python

    No handlers could be found for logger "oauth2.application"

Make sure that logging is configured to avoid this:

.. code-block:: python

    import logging
    logging.basicConfig()

"""
import logging

app_log = logging.getLogger("oauth2.application")
gen_log = logging.getLogger("oauth2.general")



"""
Provides various implementations of algorithms to generate an Access Token or
Refresh Token.
"""

import hashlib
import os
import uuid


class TokenGenerator(object):
    """
    Base class of every token generator.
    """
    def __init__(self):
        """
        Create a new instance of a token generator.
        """
        self.expires_in = {}
        self.refresh_expires_in = 0

    def create_access_token_data(self, grant_type):
        """
        Create data needed by an access token.

        :param grant_type:
        :type grant_type: str

        :return: A ``dict`` containing he ``access_token`` and the
                 ``token_type``. If the value of ``TokenGenerator.expires_in``
                 is larger than 0, a ``refresh_token`` will be generated too.
        :rtype: dict
        """
        result = {"access_token": self.generate(), "token_type": "Bearer"}

        if self.expires_in.get(grant_type, 0) > 0:
            result["refresh_token"] = self.generate()

            result["expires_in"] = self.expires_in[grant_type]

        return result

    def generate(self):
        """
        Implemented by generators extending this base class.

        :raises NotImplementedError:
        """
        raise NotImplementedError


class URandomTokenGenerator(TokenGenerator):
    """
    Create a token using ``os.urandom()``.
    """
    def __init__(self, length=40):
        self.token_length = length
        TokenGenerator.__init__(self)

    def generate(self):
        """
        :return: A new token
        :rtype: str
        """
        random_data = os.urandom(100)

        hash_gen = hashlib.new("sha512")
        hash_gen.update(random_data)

        return hash_gen.hexdigest()[:self.token_length]


class Uuid4(TokenGenerator):
    """
    Generate a token using uuid4.
    """
    def generate(self):
        """
        :return: A new token
        :rtype: str
        """
        return str(uuid.uuid4())



"""
Store adapters to read/write data to from/to mongodb using pymongo.
"""

from oauth2.store import AccessTokenStore, AuthCodeStore, ClientStore
from oauth2.datatype import AccessToken, AuthorizationCode, Client
from oauth2.error import AccessTokenNotFound, AuthCodeNotFound, \
    ClientNotFoundError
import pymongo


class MongodbStore(object):
    """
    Base class extended by all concrete store adapters.
    """

    def __init__(self, collection):
        self.collection = collection


class AccessTokenStore(AccessTokenStore, MongodbStore):
    """
    Create a new instance like this::

        from pymongo import MongoClient

        client = MongoClient('localhost', 27017)

        db = client.test_database

        access_token_store = AccessTokenStore(collection=db["access_tokens"])

    """

    def fetch_by_refresh_token(self, refresh_token):
        data = self.collection.find_one({"refresh_token": refresh_token})

        if data is None:
            raise AccessTokenNotFound

        return AccessToken(client_id=data.get("client_id"),
                           grant_type=data.get("grant_type"),
                           token=data.get("token"),
                           data=data.get("data"),
                           expires_at=data.get("expires_at"),
                           refresh_token=data.get("refresh_token"),
                           refresh_expires_at=data.get("refresh_expires_at"),
                           scopes=data.get("scopes"))

    def delete_refresh_token(self, refresh_token):
        """
        Deletes (invalidates) an old refresh token after use
        :param refresh_token: The refresh token.
        """
        self.collection.remove({"refresh_token": refresh_token})

    def fetch_existing_token_of_user(self, client_id, grant_type, user_id):
        data = self.collection.find_one({"client_id": client_id,
                                         "grant_type": grant_type,
                                         "user_id": user_id},
                                        sort=[("expires_at",
                                               pymongo.DESCENDING)])

        if data is None:
            raise AccessTokenNotFound

        return AccessToken(client_id=data.get("client_id"),
                           grant_type=data.get("grant_type"),
                           token=data.get("token"),
                           data=data.get("data"),
                           expires_at=data.get("expires_at"),
                           refresh_token=data.get("refresh_token"),
                           refresh_expires_at=data.get("refresh_expires_at"),
                           scopes=data.get("scopes"),
                           user_id=data.get("user_id"))

    def save_token(self, access_token):
        self.collection.insert({
            "client_id": access_token.client_id,
            "grant_type": access_token.grant_type,
            "token": access_token.token,
            "data": access_token.data,
            "expires_at": access_token.expires_at,
            "refresh_token": access_token.refresh_token,
            "refresh_expires_at": access_token.refresh_expires_at,
            "scopes": access_token.scopes,
            "user_id": access_token.user_id})

        return True


class AuthCodeStore(AuthCodeStore, MongodbStore):
    """
    Create a new instance like this::

        from pymongo import MongoClient

        client = MongoClient('localhost', 27017)

        db = client.test_database

        access_token_store = AuthCodeStore(collection=db["auth_codes"])

    """

    def fetch_by_code(self, code):
        code_data = self.collection.find_one({"code": code})

        if code_data is None:
            raise AuthCodeNotFound

        return AuthorizationCode(client_id=code_data.get("client_id"),
                                 code=code_data.get("code"),
                                 expires_at=code_data.get("expires_at"),
                                 redirect_uri=code_data.get("redirect_uri"),
                                 scopes=code_data.get("scopes"),
                                 data=code_data.get("data"),
                                 user_id=code_data.get("user_id"))

    def save_code(self, authorization_code):
        self.collection.insert({
            "client_id": authorization_code.client_id,
            "code": authorization_code.code,
            "expires_at": authorization_code.expires_at,
            "redirect_uri": authorization_code.redirect_uri,
            "scopes": authorization_code.scopes,
            "data": authorization_code.data,
            "user_id": authorization_code.user_id})

        return True

    def delete_code(self, code):
        """
        Deletes an authorization code after use
        :param code: The authorization code.
        """
        self.collection.remove({"code": code})


class ClientStore(ClientStore, MongodbStore):
    """
    Create a new instance like this::

        from pymongo import MongoClient

        client = MongoClient('localhost', 27017)

        db = client.test_database

        access_token_store = ClientStore(collection=db["clients"])

    """

    def fetch_by_client_id(self, client_id):
        client_data = self.collection.find_one({"identifier": client_id})

        if client_data is None:
            raise ClientNotFoundError

        return Client(
            identifier=client_data.get("identifier"),
            secret=client_data.get("secret"),
            redirect_uris=client_data.get("redirect_uris"),
            authorized_grants=client_data.get("authorized_grants"),
            authorized_response_types=client_data.get(
                "authorized_response_types"
            ))
